import { DomainResource, Resource, id, Extension, uri, Narrative } from 'fhir';
import { ResourceBuilder } from './resource-builder';
/**
 * Class representing a FHIR DomainResource builder
 */
export declare class DomainResourceBuilder extends ResourceBuilder {
    /**
     * The DomainResource that is being built
     * @protected
     * @type { DomainResource }
     * @ignore
     */
    protected resource: DomainResource;
    /**
     * Creates a new FHIR DomainResourceBuilder
     */
    constructor();
    /**
     * Sets a copy of an existing domainResource for this builder to allow for manipulation,
     * If no argument is provided, a new DomainResource will be generated
     * @param { DomainResource } domainResource - the domainResource to be set
     * @returns { DomainResourceBuilder } The builder
     * @override
     */
    use(domainResource?: DomainResource): this;
    /**
     * Returns the text-field of the DomainResource
     * @returns The text-field
     */
    getText(): Narrative | undefined;
    /**
     * Sets the text-field of the DomainResource
     * If no argument is provided, the field will be removed
     * @param { string } text - The value to be set in the text-field
     * @returns { DomainResourceBuilder } - The builder
     */
    setText(text?: Narrative): this;
    /**
     * Returns either the resource identified by the provided id from the contained array or the entire contained array, if no id is provided
     * @returns { Resource | undefined | Resource[] } The resource if it exists or the contained-array
     */
    getContained(): Resource[];
    getContained(id: id): Resource | undefined;
    /**
     * Sets the resource's contained-array to the provided array
     * If no argument is provided, the field will be removed
     * @param { Resource[] } contained The array to be set as the resource's contained array
     * @returns the Builder
     */
    setContained(contained?: Resource[]): this;
    /**
     * Adds the resource provided as an argument to the resource's contained array, if a resource with that id does not already exist in the array.
     * @param contained The resource to be added to the contained array
     * @returns the Builder
     * @throws { AlreadyExistsError } If the resource already exists
     */
    addContained(contained: Resource): this;
    /**
     * Returns either the extensions identified by the provided url from the extension array or the entire extension array, if no url is provided
     * @param { uri } [url] The url of the extensions to retrieve (if applicable)
     * @returns { Resource | undefined | Resource[] } The extensions if they exists or the extension-array
     */
    getExtension(url?: uri): Extension[];
    /**
     * Sets the resource's extension-array to the provided array
     * If no argument is provided, the field will be removed
     * @param { Extension[] } extension The array to be set as the resource's extension array
     * @returns the Builder
     */
    setExtension(extension?: Extension[]): this;
    /**
     * Adds the extension provided as an argument to the resource's extension array
     * @param { Extension } extension The extension to be added to the extension-array
     * @returns the Builder
     */
    addExtension(extension: Extension): this;
    /**
     * Returns either the modifierExtensions identified by the provided url from the modifierExtension array or the entire modifierExtension array, if no url is provided
     * @param { uri } [url] The url of the modifierExtensions to retrieve (if applicable)
     * @returns { Resource | undefined | Resource[] } The modifierExtensions if they exists or the modifierExtension-array
     */
    getModifierExtension(url?: uri): Extension[];
    /**
     * Sets the resource's modifierExtension-array to the provided array
     * If no argument is provided, the field will be removed
     * @param { Extension[] } modifierExtension The array to be set as the resource's modifierExtension array
     * @returns the Builder
     */
    setModifierExtension(modifierExtension?: Extension[]): this;
    /**
     * Adds the extension provided as an argument to the resource's modifierExtension array
     * @param { Extension } modifierExtension The modifierExtension to be added to the modifierExtension-array
     * @returns the Builder
     */
    addModifierExtension(modifierExtension: Extension): this;
    /**
     * Make a new object that implements a FHIR DomainResource
     * @returns { DomainResource } - an object that implements FHIR DomainResource
     * @override
     */
    protected make(): DomainResource;
    /**
     * Builds and returns the domainResource
     * @returns { DomainResource } - the domainResource
     * @override
     */
    build(): DomainResource;
}
