import { Resource, id, code, Meta, uri } from 'fhir';
/**
 * Class representing a FHIR Resource builder
 */
export declare class ResourceBuilder {
    /**
     * The Resource that is being built
     * @protected
     * @type { Resource }
     * @ignore
     */
    protected resource: Resource;
    /**
     * Creates a new FHIR ResourceBuilder
     */
    constructor();
    /**
     * Sets a copy of an existing resource for this builder to allow for manipulation
     * If no argument is provided, a new Resource will be generated
     * @param { Resource } resource - the resource to be set
     * @returns { ResourceBuilder } The builder
     * @override
     */
    use(resource?: Resource): this;
    /**
     * Returns the resourceType-field of the Resource
     * @returns { code } The resourceType-field
     */
    getResourceType(): code | undefined;
    /**
     * Sets the resourceType-field of the Resource
     * If no argument is provided, the field will be removed
     * @param { code } resourceType - The value to be set in the resourceType-field
     * @returns { ResourceBuilder } - The builder
     */
    setResourceType(resourceType?: code): this;
    /**
     * Returns the resource's id
     * @returns { id } The resource's id
     */
    getId(): id | undefined;
    /**
     * Sets the resource's id to the id provided as an argument
     * If no argument is provided, the field will be removed
     * @param { id } id the id to be set
     * @returns the Builder
     */
    setId(id?: id): this;
    /**
     * Returns the meta-field of the Resource
     * @returns { Meta } The meta-field
     */
    getMeta(): Meta | undefined;
    /**
     * Sets the meta-field of the Resource
     * If no argument is provided, the field will be removed
     * @param { Meta } meta - The value to be set in the meta-field
     * @returns { ResourceBuilder } - The builder
     */
    setMeta(meta?: Meta): this;
    /**
     * Returns the implicitRules-field of the Resource
     * @returns { uri } The implicitRules-field
     */
    getImplicitRules(): uri | undefined;
    /**
     * Sets the implicitRules-field of the Resource
     * If no argument is provided, the field will be removed
     * @param { uri } implicitRules - The value to be set in the implicitRules-field
     * @returns { ResourceBuilder } - The builder
     */
    setImplicitRules(implicitRules?: uri): this;
    /**
     * Returns the language-field of the Resource
     * @returns { code } The language-field
     */
    getLanguage(): code | undefined;
    /**
     * Sets the language-field of the Resource
     * If no argument is provided, the field will be removed
     * @param { string } code - The value to be set in the language-field
     * @returns { ResourceBuilder } - The builder
     */
    setLanguage(language?: code): this;
    /**
     * Returns whether or not the resource provided as an argument is in conflict with the resource used in the builder
     * @param { Resource } resource The resource to test for conflict
     * @returns { boolean } Whether it is in conflict
     */
    conflictsWith(resource: Resource): boolean;
    /**
     * Make a new object that implements a FHIR Resource
     * @returns { Resource } - an object that implements FHIR Resource
     */
    protected make(): Resource;
    /**
     * Builds and returns the resource
     * @returns { Resource } - the resource
     */
    build(): Resource;
}
