import {
  uri,
  CarePlan,
  Identifier,
  Reference,
  Period,
  canonical,
  dateTime,
  CarePlanStatus,
  CarePlanIntent,
  CarePlanActivity,
  id,
  Basic,
  Task,
  Resource,
  Questionnaire,
  ObservationDefinition,
} from 'fhir'
import { AlreadyExistsError, MissingPropertyError } from './exceptions'
import { DomainResourceBuilder } from './domain-resource-builder'
import { ReferenceBuilder } from './reference-builder'
import { TaskBuilder } from './task-builder'
import { ResourceBuilder } from './resource-builder'
import { ThresholdSetBuilder } from './threshold-set-builder'
import { CarePlanActivityBuilder } from './care-plan-activity-builder'
import utils from './utils'
import { MalformedError } from './exceptions/malformed-error'
import { TaskInputBuilder } from './task-input-builder'
import { Constants } from 'global/constants'
import { _handleIdConflicts } from 'utils/plan-definition-to-careplan-util'

/**
 * Class representing a FHIR CarePlan builder
 */
export class CarePlanBuilder extends DomainResourceBuilder {
  /**
   * The CarePlan that is being built
   * @protected
   * @type { CarePlan }
   * @ignore
   */
  protected resource: CarePlan

  /**
   * Creates a new FHIR CarePlanBuilder
   */
  public constructor() {
    super()
    this.resource = this.make()
  }

  /**
   * Sets a copy of an existing carePlan for this CarePlanBuilder to allow for manipulating, or interacting with, the
   * carePlan through this CarePlanBuilder
   * If no argument is provided, a new CarePlan will be generated
   * @param { CarePlan } carePlan - The carePlan to set
   * @returns { CarePlanBuilder } The builder
   * @override
   */
  public use(carePlan?: CarePlan): this {
    this.resource = carePlan ? { ...carePlan } : this.make()
    return this
  }

  /**
   * Checks if a Questionnaire with a specific canonical is already in the CarePlan.
   * @param canonical The canonical of the Questionnaire to check for
   * @returns { boolean } A boolean value indicating whether the Questionnaire is already there or not.
   */
  public has(canonical: canonical): boolean {
    // create copy of this cpBuilder
    const cpBuilder = new CarePlanBuilder().use(this.build())

    // test if a task is in conflict
    const task: Task = new TaskBuilder()
      .addInput(new TaskInputBuilder().setValueCanonical(canonical).build())
      .build()

    try {
      cpBuilder.addContained(task)
    } catch (e) {
      return true
    }

    // test if a thresholdset is in conflict
    const thresholdSet: Basic = new ThresholdSetBuilder()
      .setAppliesTo(canonical)
      .build()

    try {
      cpBuilder.addContained(thresholdSet)
    } catch (e) {
      return true
    }

    return false
  }

  /**
   * Adds a questionnaire to the careplan, by adding a task referencing the canonical url of the questionnaire to the
   * careplan's contained array, and adding an activity referencing the task to the careplan's activity array
   * @param { Questionnaire } questionnaire The Questionnaire to add
   * @returns { CarePlanBuilder } The builder
   */
  public add(questionnaire: Questionnaire): this {
    if (!questionnaire.url) {
      throw new MalformedError('Cannot add resource: url field is empty.')
    }
    const canonical = questionnaire.version
      ? `${questionnaire.url}|${questionnaire.version}`
      : questionnaire.url

    if (this.has(canonical)) {
      throw new AlreadyExistsError(
        'The questionnaire already exists in the CarePlan.'
      )
    }

    // Create Task
    let task: Task = utils._createTask(canonical)

    // Add Task to Contained-array
    task = utils._handleIdConflicts(task, this.resource) as Task
    this.addContained(task)

    // Create activity with ref to Task and ThresholdSet
    const activity: CarePlanActivity = utils._createActivity(task.id)

    // Add Activity to CP
    this.addActivity(activity)

    // Retrieve Questionnaire's ThresholdSet extension if it exists
    const thresholdSetExtension = questionnaire.extension?.find(extension => (
      extension.url === Constants.outcomeDefinition.thresholdSet &&
      extension.valueReference?.reference
    ))
    // Get reference to contained ThresholdSet from extension
    const thresholdSetReference = thresholdSetExtension?.valueReference?.reference ?? ''
    // Retrieve ThresholdSet from contained array
    const thresholdSet = thresholdSetReference
      ? new DomainResourceBuilder().use(questionnaire).getContained(thresholdSetReference)
      : undefined

    // add Questionnaire thresholdSet to CarePlan's contained array
    if (thresholdSet) {
      const safeThresholdSet = _handleIdConflicts(thresholdSet, this.build()) as Basic
      this.addContained(safeThresholdSet)
      this.addExtension({
        url: Constants.carePlan.thresholdSet,
        valueReference: {
          reference: `#${thresholdSet.id}`
        }
      })
    }

    // Retrieve ObservationDefinitions from Questionnaire's contained-array
    const observationDefinitions = questionnaire.contained?.filter(resource => (
      resource.resourceType === 'ObservationDefinition'
    )) as ObservationDefinition[]

    // retrieve ThresholdSets from ObservationDefinitions
    const thresholdSets = observationDefinitions?.map(obsDef => {
      // Retrieve ObservationDefinition's ThresholdSet extension if it exists
      const thresholdSetExtension = obsDef.extension?.find(extension => (
        extension.url === Constants.outcomeDefinition.thresholdSet &&
        extension.valueReference?.reference
      ))

      // Get reference to contained ThresholdSet from extension or empty string if none exist
      const thresholdSetReference = thresholdSetExtension?.valueReference?.reference ?? ''
      // Retrieve ThresholdSet from contained array
      const thresholdSet = thresholdSetReference
        ? new DomainResourceBuilder().use(obsDef).getContained(thresholdSetReference)
        : undefined
      return thresholdSet
    }).filter((resource): resource is Resource => resource !== undefined)

    // Add ObservationDefinition ThresholdSets to CarePlans contained-array
    if (thresholdSets) {
      thresholdSets.forEach(thresholdSet => {
        // Test if a contained ThresholdSet appliesTo the same resource
        if (!this.getContained().some(resource => {
          // If contained resource isn't a basic, return
          if (resource.resourceType !== 'Basic') return false

          // Retrieve appliesTo for contained or the string 'contained' if appliesTo is undefined
          const contained = resource as Basic
          const containedAppliesTo = (contained.extension?.find(ext => (
            ext.url === Constants.thresholdSet.appliesTo && ext.valueCanonical
          ))?.valueCanonical ?? 'contained')
          
          // Retrieve appliesTo for candidate or the string 'candidate' if appliesTo is undefined
          const candidate = thresholdSet as Basic
          const candidateAppliesTo = (candidate.extension?.find(ext => (
            ext.url === Constants.thresholdSet.appliesTo && ext.valueCanonical
          ))?.valueCanonical ?? 'candidate')

          return containedAppliesTo === candidateAppliesTo
        })) {
          const safeThresholdSet = _handleIdConflicts(thresholdSet, this.build()) as Basic
          this.addContained(safeThresholdSet)
          this.addExtension({
            url: Constants.carePlan.thresholdSet,
            valueReference: {
              reference: `#${thresholdSet.id}`
            }
          })
        }
      })
    }

    return this
  }

  /**
   * Removes a Questionnaire from the CarePlan based on the provided canonical reference
   * @param canonical
   */
  public remove(canonical: canonical): this {
    // find Task referencing questionnaire through canonical
    const task = this.getContained().find(contained => {
      if (contained.resourceType !== 'Task') return false
      const task = contained as Task
      if (!task.input || task.input.length === 0) {
        return false
      }
      let found = false
      task.input.forEach(input => {
        if (input.valueCanonical === canonical) found = true
      })
      return found
    })
    if (!task) return this

    // find ThresholdSet referencing questionnaire through canonical
    const thresholdSet = this.getContained().find(contained => {
      if (contained.resourceType !== 'Basic') return false
      const tsBuilder = new ThresholdSetBuilder().use(contained as Basic)
      if (tsBuilder.getAppliesTo() === canonical) return true
      return false
    })

    // if thresholdSet was found, remove it's reference from CarePlan.extension
    if (thresholdSet) {
      const thresholdSetExtension = this
        .getExtension("https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100728833/CarePlan+thresholdset+extension")
        .find(extension => {
          if (!extension.valueReference || !extension.valueReference.reference) return false
          return extension.valueReference.reference === `#${thresholdSet.id}`
        })
      this.setExtension(this.getExtension().filter(extension => extension !== thresholdSetExtension))
    }

    // remove Task and ThresholdSet
    this.setContained(
      this.getContained().filter(
        contained => contained !== task && contained !== thresholdSet
      )
    )

    // find CarePlanActivity
    const act = this.getActivity().find(activity => {
      if (!activity.reference) return false
      return activity.reference.reference === `#${task.id}`
    })
    if (!act) return this

    // remove CarePlanActivity
    this.setActivity(this.getActivity().filter(activity => activity !== act))

    return this
  }

  /**
   * Adds the resource provided as an argument to the resource's contained array, if a resource that is in conflict does not already exist in the array.
   * @param contained The resource to be added to the contained array
   * @returns { CarePlanBuilder } the Builder
   * @throws { AlreadyExistsError } If the resource already exists
   * @override
   */
  public addContained(resource: Resource): this {
    this.getContained().forEach(contained => {
      if (this.containedConflictsWith(contained, resource)) {
        if (resource.id) {
          throw new AlreadyExistsError(
            'A resource that is in conflict with the provided resource already exists in the contained-array',
            resource.id
          )
        } else {
          throw new AlreadyExistsError(
            'A resource that is in conflict with the provided resource already exists in the contained-array'
          )
        }
      }
    })
    super.addContained(resource)
    return this
  }
  /**
   * Returns either the identifier identified by the provided system from the identifier-array or the entire identifier array, if no system is provided
   * @returns { Identifier | undefined | Identifier[] } The identifier if it exists or the identifier-array
   */
  public getIdentifier(): Identifier[]
  public getIdentifier(system: uri): Identifier | undefined
  public getIdentifier(system?: uri): Identifier | undefined | Identifier[] {
    if (system) {
      return this.getIdentifier().find(
        identifier => identifier.system === system
      )
    }
    return this.resource.identifier || []
  }

  /**
   * Replaces the identifier array with the provided array
   * If no argument is provided, the field will be removed
   * @param { Identifier[] } identifiers - the identifier to be set
   * @returns { CarePlanBuilder } - this CarePlanBuilder
   */
  public setIdentifier(identifier?: Identifier[]): this {
    identifier
      ? (this.resource.identifier = identifier)
      : delete this.resource.identifier
    return this
  }

  /**
   * Appends an identifier to the carePlan, if an identifier within the system does not already exist
   * @param { Identifier } identifier - the identifier to be appended
   * @returns { CarePlanBuilder } - this CarePlanBuilder
   * @throws { MissingPropertyError } When identifier is missing necessary properties
   * @throws { AlreadyExistsError } When an identifier in that system already exists in the CarePlan
   */
  public addIdentifier(identifier: Identifier): this {
    if (!identifier.system || !identifier.value) {
      throw new MissingPropertyError(
        'Identifiers must have both a system and a value.'
      )
    }
    if (
      this.getIdentifier().find(ident => ident.system === identifier.system)
    ) {
      throw new AlreadyExistsError(
        'An identifier with that system, already exists in the identifier-array'
      )
    }
    this.setIdentifier([...this.getIdentifier(), identifier])
    return this
  }

  /**
   * Returns all plan definition canonical references for carePlan
   * @returns { canonical[] } - the canonicals
   */
  public getInstantiatesCanonical(): canonical[] {
    return this.resource.instantiatesCanonical || []
  }

  /**
   * Replaces the instantiatesCanonical array with the provided array
   * If no argument is provided, the field will be removed
   * @param { canonical[] } canonical - the array to be set
   * @returns { CarePlanBuilder } - this CarePlanBuilder
   */
  public setInstantiatesCanonical(canonical?: canonical[]): this {
    canonical
      ? (this.resource.instantiatesCanonical = canonical)
      : delete this.resource.instantiatesCanonical
    return this
  }

  /**
   * Appends a plan definition canonical reference to the carePlan, if it does not already exist
   * @param { canonical } canonical - the canonical reference to be appended
   * @returns { CarePlanBuilder } - this CarePlanBuilder
   * @throws { AlreadyExistsError } When the canonical already exists on the CarePlan
   */
  public addInstantiatesCanonical(canonical: canonical): this {
    if (this.getInstantiatesCanonical().find(canon => canon === canonical)) {
      throw new AlreadyExistsError(
        'That canonical already exists in the instantiatesCanonical-array'
      )
    }
    this.setInstantiatesCanonical([
      ...this.getInstantiatesCanonical(),
      canonical,
    ])
    return this
  }

  /**
   * Returns the carePlan's status field
   * @returns { code } - the status of the carePlan
   */
  public getStatus(): CarePlanStatus {
    return this.resource.status
  }

  /**
   * Sets the carePlan's status field
   * If no argument is provided, the field will be removed
   * @param { CarePlanStatus } status - the status to be set. Must be one of: draft | active | suspended | completed | entered-in-error | cancelled | unknown
   * @returns { CarePlanBuilder } - The builder
   */
  public setStatus(status: CarePlanStatus): this {
    this.resource.status = status
    return this
  }

  /**
   * Returns the carePlan's intent field. This field is contrained to 'order'
   * @returns { intent } - the intent of the carePlan
   */
  public getIntent(): CarePlanIntent {
    return this.resource.intent
  }

  /**
   * Sets the carePlan's intent-field
   * If no argument is provided, the field will be removed
   * @param { CarePlanIntent } intent - the intent to be set
   * @returns { CarePlanBuilder } - The builder
   */
  public setIntent(intent: CarePlanIntent): this {
    this.resource.intent = intent
    return this
  }

  /**
   * Returns the carplan's title field
   * @return { string } the title of the carePlan
   */
  public getTitle(): string | undefined {
    return this.resource.title
  }

  /**
   * Sets the carePlan's title field.
   * If no argument is provided, the field will be removed
   * @param { string } title - the title of the carePlan
   * @return { CarePlanBuilder } this CarePlanBuilder
   */
  public setTitle(title?: string): this {
    title ? (this.resource.title = title) : delete this.resource.title
    return this
  }

  /**
   * Returns the carePlan's subject
   * @return { Reference } subject - the subject of the carePlan
   */
  public getSubject(): Reference {
    return this.resource.subject
  }

  /**
   * Sets the carePlan's subject field
   * If no argument is provided, the field will be removed
   * @param { Reference } subject - the subject which the carePlan refereces
   * @return { CarePlanBuilder } this CarePlanBuilder
   */
  public setSubject(subject?: Reference): this {
    subject ? (this.resource.subject = subject) : delete this.resource.subject
    return this
  }

  /**
   * Returns the carePlan's period
   * @returns { Period } period - the period within which the carePlan is active
   */
  public getPeriod(): Period | undefined {
    return this.resource.period
  }

  /**
   * Sets the carePlan's period field
   * If no argument is provided, the field will be removed
   * @param { Period } period - the period within which the carePlan is active
   * @return { CarePlanBuilder } this CarePlanBuilder
   */
  public setPeriod(period?: Period): this {
    period ? (this.resource.period = period) : delete this.resource.period
    return this
  }

  /**
   * Returns the carePlan's encounter
   * @returns { Reference } encounter - the encounter to which the carePlan is related.
   */
  public getEncounter(): Reference {
    const encounter = this.resource.encounter as Reference
    return encounter
  }

  /**
   * Sets the carePlan's encounter field
   * If no argument is provided, the field will be removed
   * @param { Reference } encounter - the encounter to which the carePlan is related
   * @return { CarePlanBuilder } this CarePlanBuilder
   */
  public setEncounter(encounter?: Reference): this {
    encounter
      ? (this.resource.encounter = encounter)
      : delete this.resource.encounter
    return this
  }

  /**
   * Returns the carePlan's created field
   * If no argument is provided, the field will be removed
   * @returns { dateTime } created - the date the carePlan's was created
   */
  public getCreated(): dateTime {
    const created = this.resource.created as dateTime
    return created
  }

  /**
   * Sets the carePlan's created field
   * If no argument is provided, the field will be removed
   * @param { dateTime } created - the date the carePlan's was created
   * @returns { CarePlanBuilder } this CarePlanBuilder
   */
  public setCreated(created?: dateTime): this {
    created ? (this.resource.created = created) : delete this.resource.created
    return this
  }
  /**
   * Returns the carePlan's author
   * @return { Reference } author - the author of the carePlan
   */
  public getAuthor(): Reference {
    const author = this.resource.author as Reference
    return author
  }

  /**
   * Sets the carePlan's author field
   * If no argument is provided, the field will be removed
   * @param { Reference } author - the author which the carePlan refereces
   * @return { CarePlanBuilder } The builder
   */
  public setAuthor(author?: Reference): this {
    author ? (this.resource.author = author) : delete this.resource.author
    return this
  }

  /**
   * Returns an array of references to conditions, that this carePlan addresses
   * @return { Reference[] } - the array of references
   */
  public getAddresses(): Reference[] {
    return this.resource.addresses || []
  }

  /**
   * Replace the array of references to conditions, that this carePlan addresses with the provided array
   * If no argument is provided, the field will be removed
   * @param { Reference[] } addresses The array of references
   * @returns { CarePlanBuilder } - The builder
   */
  public setAddresses(addresses?: Reference[]): this {
    if (!addresses) delete this.resource.addresses
    else {
      addresses.forEach(reference => {
        if (
          !reference.reference &&
          (!reference.identifier ||
            (!reference.identifier.system || !reference.identifier.value))
        ) {
          throw new MissingPropertyError(
            'All references must have either a literal reference or an identifier with both system and value.'
          )
        }
      })
      this.resource.addresses = addresses
    }

    return this
  }

  /**
   * Appends a reference to a condition, that this carePlan addresses
   * @param { Reference } addresses - the reference to be appended
   * @returns { CarePlanBuilder } - this CarePlanBuilder
   * @throws { MissingPropertyError } When the Reference is missing necessary properties
   */
  public addAddresses(reference: Reference): this {
    if (this.resource.addresses && this.resource.addresses.length > 0) {
      if (
        this.resource.addresses.find(ref =>
          new ReferenceBuilder().use(ref).conflictsWith(reference)
        )
      ) {
        throw new AlreadyExistsError(
          'The reference is in conflict with one of the existing references in the references-array'
        )
      }
    }
    this.setAddresses([...this.getAddresses(), reference])
    return this
  }

  /**
   * Returns either the carePlanActivity identified by the provided id from the activity-array or the entire activity-array, if no id is provided
   * @returns { CarePlanActivity[] } The carePlanActivity identified by the provided id or the activity-array
   */
  public getActivity(): CarePlanActivity[]
  public getActivity(id: id): CarePlanActivity | undefined
  public getActivity(
    id?: id
  ): CarePlanActivity | undefined | CarePlanActivity[] {
    if (id) {
      return this.getActivity().find(activity => activity.id === id)
    }
    return this.resource.activity || []
  }

  /**
   * Sets the activity-array to the provided array
   * If no argument is provided, the field will be removed
   * @param { CarePlanActivity[] } activity The array to be set as the action-array
   * @returns the Builder
   */
  public setActivity(activity?: CarePlanActivity[]): this {
    activity
      ? (this.resource.activity = activity)
      : delete this.resource.activity
    return this
  }

  /**
   * Adds the activity provided as an argument to the resource's activity-array
   * @param { CarePlanActivity } activity The activity to be added to the activity-array
   * @returns the Builder
   * @throws { AlreadyExistsError } If the activity is in conflict with an acitivty already existing in the activity array
   */
  public addActivity(activity: CarePlanActivity): this {
    if (
      this.getActivity().find(act =>
        new CarePlanActivityBuilder().use(act).conflictsWith(activity)
      )
    ) {
      throw new AlreadyExistsError(
        'The carePlanActivity is in conflict with one of the existing activities in the activity-array'
      )
    }
    this.setActivity([...this.getActivity(), activity])
    return this
  }

  /**
   * Returns whether the two provided resources are in conflict
   * @param { Resource } contained First provided resource
   * @param { Resource } resource second provided resource
   * @returns { boolean } Whether the two resources are in conflict
   * @ignore
   */
  private containedConflictsWith(
    contained: Resource,
    resource: Resource
  ): boolean {
    let conflict = new ResourceBuilder().use(resource).conflictsWith(contained)
    switch (resource.resourceType) {
      case 'Task':
        conflict =
          contained.resourceType === 'Task' &&
          new TaskBuilder()
            .use(resource as Task)
            .conflictsWith(contained as Task)
        break
      case 'Basic':
        conflict =
          contained.resourceType === 'Basic' &&
          new ThresholdSetBuilder()
            .use(resource as Basic)
            .conflictsWith(contained as Basic)
        break
    }
    return conflict
  }

  /**
   * Returns the activity from the activity-array that is in conflict with the provided activity, if one exists
   * @param { CarePlanActivity } activity The activity to test for conflicts
   * @returns { CarePlanActivity } The activity that is in conflict with the provided activity
   *
   */
  public getConflictingActivity(
    activity: CarePlanActivity
  ): CarePlanActivity | undefined {
    return this.getActivity().find(act =>
      new CarePlanActivityBuilder().use(act).conflictsWith(activity)
    )
  }

  /**
   * Returns the resource from the contained-array, that is in conflict with the provided resource, if one exists
   * @param { Resource } activity The activity to test for conflicts
   * @returns { Resource } The resource that is in conflict with the provided activity
   *
   */
  public getConflictingContained(resource: Resource): Resource | undefined {
    return this.getContained().find(contained =>
      this.containedConflictsWith(contained, resource)
    )
  }

  /**
   * Creates an ISO-string for current time
   * @returns { string } An ISO-string for current time
   * @ignore
   */
  /* istanbul ignore next */
  public _generateIsoDate(): string {
    return utils._generateIsoDate()
  }

  /**
   * Make a new object that implements a FHIR CarePlan
   * @returns { CarePlan } - an object that implements FHIR CarePlan
   * @ignore
   * @override
   */
  protected make(): CarePlan {
    return {
      resourceType: 'CarePlan',
      created: this._generateIsoDate(),
    } as CarePlan
  }

  /**
   * Builds and returns the carePlan
   * @returns { CarePlan } - the carePlan
   * @override
   */
  public build(): CarePlan {
    return this.resource
  }
}
