import { CodeableConcept, Coding, uri } from 'fhir'
import { ElementBuilder } from './element-builder'

/** A class representing a FHIR CodeableConceptBuilder */
export class CodeableConceptBuilder extends ElementBuilder {
  /**
   * The CodeableConcept being built
   * @private
   * @type { CodeableConcept }
   * @ignore
   */
  protected element: CodeableConcept

  /**
   * Creates a new CodeableConceptBuilder
   */
  public constructor() {
    super()
    this.element = this.make()
  }

  /**
   * Sets a copy of an existing codeableConcept for this builder to allow for manipulation,
   * If no argument is provided, a new CodeableConcept will be generated
   * @param { CodeableConcept } codeableConcept - the codeableConcept to be set
   * @returns { CodeableConceptBuilder } The builder
   * @override
   */
  public use(codeableConcept?: CodeableConcept): this {
    this.element = codeableConcept ? { ...codeableConcept } : this.make()
    return this
  }

  /**
   * Returns the text-field of the CodeableConcept
   * @returns The text-field
   */
  public getText(): string | undefined {
    return this.element.text
  }

  /**
   * Sets the text-field of the CodeableConcept
   * If no argument is provided, the field will be removed
   * @param { string } text - The value to be set in the text-field
   * @returns { CodeableConceptBuilder } - This CodeableConceptBuilder
   */
  public setText(text?: string): this {
    text ? (this.element.text = text) : delete this.element.text
    return this
  }

  /**
   * Returns either the coding identified by the provided system from the coding-array or the entire coding array, if no id is provided
   * @param { uri } [system] The system of the coding to retrieve (if applicable)
   * @returns { Coding | undefined | Coding[] } The coding or the coding-array
   */
  public getCoding(): Coding[]
  public getCoding(system: uri): Coding | undefined
  public getCoding(system?: uri): Coding | undefined | Coding[] {
    if (system) {
      return this.getCoding().find(coding => coding.system === system)
    }
    return this.element.coding || []
  }

  /**
   * Sets the resource's coding-array to the provided array
   * If no argument is provided, the field will be removed
   * @param { Coding[] } codings The array to be set as the resource's coding array
   * @returns the Builder
   */
  public setCoding(coding?: Coding[]): this {
    coding ? (this.element.coding = coding) : delete this.element.coding
    return this
  }

  /**
   * Adds a coding to the CodeableConcept
   * @param { Coding } coding - The coding to add
   * @returns { CodeableConceptBuilder} The builder
   */
  public addCoding(coding: Coding): this {
    this.setCoding([...this.getCoding(), coding])
    return this
  }

  /**
   * Creates a new CodeableConcept
   * @returns { CodeableConcept } The codeableConcept
   * @override
   */
  protected make(): CodeableConcept {
    return {} as CodeableConcept
  }

  /**
   * Builds and returns the CodeableConcept
   * @override
   * @returns { CodeableConcept } - the CodeableConcept
   */
  public build(): CodeableConcept {
    return this.element
  }
}
