import { Coding, code } from 'fhir'
import { ElementBuilder } from './element-builder'

/** A class representing a FHIR CodingBuilder */
export class CodingBuilder extends ElementBuilder {
  /**
   * The Coding being built
   * @private
   * @type { Coding }
   * @ignore
   */
  protected element: Coding

  /**
   * Creates a new CodingBuilder
   */
  public constructor() {
    super()
    this.element = this.make()
  }

  /**
   * Sets a copy of an existing coding for this builder to allow for manipulation,
   * If no argument is provided, a new Coding will be generated
   * @param { Coding } coding - the coding to be set
   * @returns { CodingBuilder } The builder
   * @override
   */
  public use(coding?: Coding): this {
    this.element = coding ? { ...coding } : this.make()
    return this
  }

  /**
   * Returns the system-field of the Coding
   * @returns { string } The system-field
   */
  public getSystem(): string | undefined {
    return this.element.system
  }

  /**
   * Sets the system-field of the Coding
   * If no argument is provided, the field will be removed
   * @param { string } system - The value to be set in the system-field
   * @returns { CodingBuilder } - This CodingBuilder
   */
  public setSystem(system?: string): this {
    system ? (this.element.system = system) : delete this.element.system
    return this
  }

  /**
   * Returns the version-field of the Coding
   * @returns { string } The version-field
   */
  public getVersion(): string | undefined {
    return this.element.version
  }

  /**
   * Sets the version-field of the Coding
   * If no argument is provided, the field will be removed
   * @param { string } version - The value to be set in the version-field
   * @returns { CodingBuilder } - This CodingBuilder
   */
  public setVersion(version?: string): this {
    version ? (this.element.version = version) : delete this.element.version
    return this
  }

  /**
   * Returns the code-field of the Coding
   * @returns { code } The code-field
   */
  public getCode(): code | undefined {
    return this.element.code
  }

  /**
   * Sets the code-field of the Coding
   * If no argument is provided, the field will be removed
   * @param { code } code - The value to be set in the code-field
   * @returns { CodingBuilder } - This CodingBuilder
   */
  public setCode(code?: code): this {
    code ? (this.element.code = code) : delete this.element.code
    return this
  }

  /**
   * Returns the display-field of the Coding
   * @returns { string } The display-field
   */
  public getDisplay(): string | undefined {
    return this.element.display
  }

  /**
   * Sets the display-field of the Coding
   * If no argument is provided, the field will be removed
   * @param { string } display - The value to be set in the display-field
   * @returns { CodingBuilder } - This CodingBuilder
   */
  public setDisplay(display?: string): this {
    display ? (this.element.display = display) : delete this.element.display
    return this
  }

  /**
   * Returns the userSelected-field of the Coding
   * @returns { boolean } The userSelected-field
   */
  public getUserSelected(): boolean | undefined {
    return this.element.userSelected
  }

  /**
   * Sets the userSelected-field of the Coding
   * If no argument is provided, the field will be removed
   * @param { boolean } userSelected - The value to be set in the userSelected-field
   * @returns { CodingBuilder } - This CodingBuilder
   */
  public setUserSelected(userSelected?: boolean): this {
    userSelected
      ? (this.element.userSelected = userSelected)
      : delete this.element.userSelected
    return this
  }

  /**
   * Creates a new Coding
   * @returns { Coding } The coding
   * @override
   */
  protected make(): Coding {
    return {} as Coding
  }

  /**
   * Builds and returns the Coding
   * @returns { Coding } - the Coding
   * @override
   */
  public build(): Coding {
    return this.element
  }
}
