import {
  Extension,
  uri,
  Reference,
  base64Binary,
  canonical,
  code,
  date,
  dateTime,
  decimal,
  id,
  instant,
  integer,
  markdown,
  oid,
  positiveInt,
  time,
  unsignedInt,
  url,
  uuid,
  Address,
  Age,
  Annotation,
  Attachment,
  CodeableConcept,
  Coding,
  ContactPoint,
  Count,
  Distance,
  Duration,
  HumanName,
  Identifier,
  Money,
  Period,
  Quantity,
  Ratio,
  SampledData,
  Signature,
  Timing,
  Meta,
  Range,
} from 'fhir'
import { ElementBuilder } from './element-builder'

/** Class representing a FHIR  */
export class ExtensionBuilder extends ElementBuilder {
  /**
   * The Extension that is being built
   * @protected
   * @type { Extension }
   * @ignore
   */
  protected element: Extension

  /**
   * Create a new FHIR ExtensionBuilder
   */
  public constructor() {
    super()
    this.element = this.make()
  }

  /**
   * Sets a copy of an already existing extension as the element to build
   * If no argument is provided, a new Extension will be generated
   * @returns { ExtensionBuilder } The builder
   * @override
   */
  public use(extension?: Extension): this {
    this.element = extension ? { ...extension } : this.make()
    return this
  }

  /**
   * Returns the url-field of the CarePlanActivity if it is set
   * @returns { Extension } The url-field
   */
  public getUrl(): uri | undefined {
    return this.element.url
  }

  /**
   * Sets the url-field of the CarePlanActivity with the provided uri
   * If no argument is provided, the field will be removed
   * @returns { CarePlanActivityBuilder } The builder
   */
  public setUrl(url?: uri): this {
    url ? (this.element.url = url) : delete this.element.url
    return this
  }

  /**
   * Returns the valueBase64Binary-field of the Extension
   * @returns The valueBase64Binary-field
   */
  public getValueBase64Binary(): base64Binary | undefined {
    return this.element.valueBase64Binary
  }

  /**
   * Sets the valueBase64Binary-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { base64Binary } valueBase64Binary - The value to be set in the valueBase64Binary-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueBase64Binary(valueBase64Binary?: base64Binary): this {
    valueBase64Binary
      ? (this.element.valueBase64Binary = valueBase64Binary)
      : delete this.element.valueBase64Binary
    return this
  }

  /**
   * Returns the valueBoolean-field of the Extension
   * @returns The valueBoolean-field
   */
  public getValueBoolean(): boolean | undefined {
    return this.element.valueBoolean
  }

  /**
   * Sets the valueBoolean-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { boolean } valueBoolean - The value to be set in the valueBoolean-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueBoolean(valueBoolean?: boolean): this {
    typeof valueBoolean !== 'undefined'
      ? (this.element.valueBoolean = valueBoolean)
      : delete this.element.valueBoolean
    return this
  }

  /**
   * Returns the valueCanonical-field of the Extension
   * @returns The valueCanonical-field
   */
  public getValueCanonical(): canonical | undefined {
    return this.element.valueCanonical
  }

  /**
   * Sets the valueCanonical-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { canonical } valueCanonical - The value to be set in the valueCanonical-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueCanonical(valueCanonical?: canonical): this {
    valueCanonical
      ? (this.element.valueCanonical = valueCanonical)
      : delete this.element.valueCanonical
    return this
  }

  /**
   * Returns the valueCode-field of the Extension
   * @returns The valueCode-field
   */
  public getValueCode(): code | undefined {
    return this.element.valueCode
  }

  /**
   * Sets the valueCode-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { code } valueCode - The value to be set in the valueCode-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueCode(valueCode?: code): this {
    valueCode
      ? (this.element.valueCode = valueCode)
      : delete this.element.valueCode
    return this
  }

  /**
   * Returns the valueDate-field of the Extension
   * @returns The valueDate-field
   */
  public getValueDate(): date | undefined {
    return this.element.valueDate
  }

  /**
   * Sets the valueDate-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { date } valueDate - The value to be set in the valueDate-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueDate(valueDate?: date): this {
    valueDate
      ? (this.element.valueDate = valueDate)
      : delete this.element.valueDate
    return this
  }

  /**
   * Returns the valueDateTime-field of the Extension
   * @returns The valueDateTime-field
   */
  public getValueDateTime(): dateTime | undefined {
    return this.element.valueDateTime
  }

  /**
   * Sets the valueDateTime-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { dateTime } valueDateTime - The value to be set in the valueDateTime-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueDateTime(valueDateTime?: dateTime): this {
    valueDateTime
      ? (this.element.valueDateTime = valueDateTime)
      : delete this.element.valueDateTime
    return this
  }

  /**
   * Returns the valueDecimal-field of the Extension
   * @returns The valueDecimal-field
   */
  public getValueDecimal(): decimal | undefined {
    return this.element.valueDecimal
  }

  /**
   * Sets the valueDecimal-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { decimal } valueDecimal - The value to be set in the valueDecimal-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueDecimal(valueDecimal?: decimal): this {
    valueDecimal
      ? (this.element.valueDecimal = valueDecimal)
      : delete this.element.valueDecimal
    return this
  }

  /**
   * Returns the valueId-field of the Extension
   * @returns The valueId-field
   */
  public getValueId(): id | undefined {
    return this.element.valueId
  }

  /**
   * Sets the valueId-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { id } valueId - The value to be set in the valueId-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueId(valueId?: id): this {
    valueId ? (this.element.valueId = valueId) : delete this.element.valueId
    return this
  }

  /**
   * Returns the valueInstant-field of the Extension
   * @returns The valueInstant-field
   */
  public getValueInstant(): instant | undefined {
    return this.element.valueInstant
  }

  /**
   * Sets the valueInstant-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { instant } valueInstant - The value to be set in the valueInstant-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueInstant(valueInstant?: instant): this {
    valueInstant
      ? (this.element.valueInstant = valueInstant)
      : delete this.element.valueInstant
    return this
  }

  /**
   * Returns the valueInteger-field of the Extension
   * @returns The valueInteger-field
   */
  public getValueInteger(): integer | undefined {
    return this.element.valueInteger
  }

  /**
   * Sets the valueInteger-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { integer } valueInteger - The value to be set in the valueInteger-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueInteger(valueInteger?: integer): this {
    valueInteger
      ? (this.element.valueInteger = valueInteger)
      : delete this.element.valueInteger
    return this
  }

  /**
   * Returns the valueMarkdown-field of the Extension
   * @returns The valueMarkdown-field
   */
  public getValueMarkdown(): markdown | undefined {
    return this.element.valueMarkdown
  }

  /**
   * Sets the valueMarkdown-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { markdown } valueMarkdown - The value to be set in the valueMarkdown-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueMarkdown(valueMarkdown?: markdown): this {
    valueMarkdown
      ? (this.element.valueMarkdown = valueMarkdown)
      : delete this.element.valueMarkdown
    return this
  }

  /**
   * Returns the valueOid-field of the Extension
   * @returns The valueOid-field
   */
  public getValueOid(): oid | undefined {
    return this.element.valueOid
  }

  /**
   * Sets the valueOid-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { oid } valueOid - The value to be set in the valueOid-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueOid(valueOid?: oid): this {
    valueOid ? (this.element.valueOid = valueOid) : delete this.element.valueOid
    return this
  }

  /**
   * Returns the valuePositiveInt-field of the Extension
   * @returns The valuePositiveInt-field
   */
  public getValuePositiveInt(): positiveInt | undefined {
    return this.element.valuePositiveInt
  }

  /**
   * Sets the valuePositiveInt-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { positiveInt } valuePositiveInt - The value to be set in the valuePositiveInt-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValuePositiveInt(valuePositiveInt?: positiveInt): this {
    valuePositiveInt
      ? (this.element.valuePositiveInt = valuePositiveInt)
      : delete this.element.valuePositiveInt
    return this
  }

  /**
   * Returns the valueString-field of the Extension
   * @returns The valueString-field
   */
  public getValueString(): string | undefined {
    return this.element.valueString
  }

  /**
   * Sets the valueString-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { string } valueString - The value to be set in the valueString-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueString(valueString?: string): this {
    valueString
      ? (this.element.valueString = valueString)
      : delete this.element.valueString
    return this
  }

  /**
   * Returns the valueTime-field of the Extension
   * @returns The valueTime-field
   */
  public getValueTime(): time | undefined {
    return this.element.valueTime
  }

  /**
   * Sets the valueTime-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { time } valueTime - The value to be set in the valueTime-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueTime(valueTime?: time): this {
    valueTime
      ? (this.element.valueTime = valueTime)
      : delete this.element.valueTime
    return this
  }

  /**
   * Returns the valueUnsignedInt-field of the Extension
   * @returns The valueUnsignedInt-field
   */
  public getValueUnsignedInt(): unsignedInt | undefined {
    return this.element.valueUnsignedInt
  }

  /**
   * Sets the valueUnsignedInt-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { unsignedInt } valueUnsignedInt - The value to be set in the valueUnsignedInt-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueUnsignedInt(valueUnsignedInt?: unsignedInt): this {
    valueUnsignedInt
      ? (this.element.valueUnsignedInt = valueUnsignedInt)
      : delete this.element.valueUnsignedInt
    return this
  }

  /**
   * Returns the valueUri-field of the Extension
   * @returns The valueUri-field
   */
  public getValueUri(): uri | undefined {
    return this.element.valueUri
  }

  /**
   * Sets the valueUri-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { uri } valueUri - The value to be set in the valueUri-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueUri(valueUri?: uri): this {
    valueUri ? (this.element.valueUri = valueUri) : delete this.element.valueUri
    return this
  }

  /**
   * Returns the valueUrl-field of the Extension
   * @returns The valueUrl-field
   */
  public getValueUrl(): url | undefined {
    return this.element.valueUrl
  }

  /**
   * Sets the valueUrl-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { url } valueUrl - The value to be set in the valueUrl-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueUrl(valueUrl?: url): this {
    valueUrl ? (this.element.valueUrl = valueUrl) : delete this.element.valueUrl
    return this
  }

  /**
   * Returns the valueUuid-field of the Extension
   * @returns The valueUuid-field
   */
  public getValueUuid(): uuid | undefined {
    return this.element.valueUuid
  }

  /**
   * Sets the valueUuid-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { uuid } valueUuid - The value to be set in the valueUuid-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueUuid(valueUuid?: uuid): this {
    valueUuid
      ? (this.element.valueUuid = valueUuid)
      : delete this.element.valueUuid
    return this
  }

  /**
   * Returns the valueAddress-field of the Extension
   * @returns The valueAddress-field
   */
  public getValueAddress(): Address | undefined {
    return this.element.valueAddress
  }

  /**
   * Sets the valueAddress-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Address } valueAddress - The value to be set in the valueAddress-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueAddress(valueAddress?: Address): this {
    valueAddress
      ? (this.element.valueAddress = valueAddress)
      : delete this.element.valueAddress
    return this
  }

  /**
   * Returns the valueAge-field of the Extension
   * @returns The valueAge-field
   */
  public getValueAge(): Age | undefined {
    return this.element.valueAge
  }

  /**
   * Sets the valueAge-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Age } valueAge - The value to be set in the valueAge-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueAge(valueAge?: Age): this {
    valueAge ? (this.element.valueAge = valueAge) : delete this.element.valueAge
    return this
  }

  /**
   * Returns the valueAnnotation-field of the Extension
   * @returns The valueAnnotation-field
   */
  public getValueAnnotation(): Annotation | undefined {
    return this.element.valueAnnotation
  }

  /**
   * Sets the valueAnnotation-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Annotation } valueAnnotation - The value to be set in the valueAnnotation-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueAnnotation(valueAnnotation?: Annotation): this {
    valueAnnotation
      ? (this.element.valueAnnotation = valueAnnotation)
      : delete this.element.valueAnnotation
    return this
  }

  /**
   * Returns the valueAttachment-field of the Extension
   * @returns The valueAttachment-field
   */
  public getValueAttachment(): Attachment | undefined {
    return this.element.valueAttachment
  }

  /**
   * Sets the valueAttachment-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Attachment } valueAttachment - The value to be set in the valueAttachment-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueAttachment(valueAttachment?: Attachment): this {
    valueAttachment
      ? (this.element.valueAttachment = valueAttachment)
      : delete this.element.valueAttachment
    return this
  }

  /**
   * Returns the valueCodeableConcept-field of the Extension
   * @returns The valueCodeableConcept-field
   */
  public getValueCodeableConcept(): CodeableConcept | undefined {
    return this.element.valueCodeableConcept
  }

  /**
   * Sets the valueCodeableConcept-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { CodeableConcept } valueCodeableConcept - The value to be set in the valueCodeableConcept-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueCodeableConcept(valueCodeableConcept?: CodeableConcept): this {
    valueCodeableConcept
      ? (this.element.valueCodeableConcept = valueCodeableConcept)
      : delete this.element.valueCodeableConcept
    return this
  }

  /**
   * Returns the valueCoding-field of the Extension
   * @returns The valueCoding-field
   */
  public getValueCoding(): Coding | undefined {
    return this.element.valueCoding
  }

  /**
   * Sets the valueCoding-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Coding } valueCoding - The value to be set in the valueCoding-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueCoding(valueCoding?: Coding): this {
    valueCoding
      ? (this.element.valueCoding = valueCoding)
      : delete this.element.valueCoding
    return this
  }

  /**
   * Returns the valueContactPoint-field of the Extension
   * @returns The valueContactPoint-field
   */
  public getValueContactPoint(): ContactPoint | undefined {
    return this.element.valueContactPoint
  }

  /**
   * Sets the valueContactPoint-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { ContactPoint } valueContactPoint - The value to be set in the valueContactPoint-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueContactPoint(valueContactPoint?: ContactPoint): this {
    valueContactPoint
      ? (this.element.valueContactPoint = valueContactPoint)
      : delete this.element.valueContactPoint
    return this
  }

  /**
   * Returns the valueCount-field of the Extension
   * @returns The valueCount-field
   */
  public getValueCount(): Count | undefined {
    return this.element.valueCount
  }

  /**
   * Sets the valueCount-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Count } valueCount - The value to be set in the valueCount-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueCount(valueCount?: Count): this {
    valueCount
      ? (this.element.valueCount = valueCount)
      : delete this.element.valueCount
    return this
  }

  /**
   * Returns the valueDistance-field of the Extension
   * @returns The valueDistance-field
   */
  public getValueDistance(): Distance | undefined {
    return this.element.valueDistance
  }

  /**
   * Sets the valueDistance-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Distance } valueDistance - The value to be set in the valueDistance-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueDistance(valueDistance?: Distance): this {
    valueDistance
      ? (this.element.valueDistance = valueDistance)
      : delete this.element.valueDistance
    return this
  }

  /**
   * Returns the valueDuration-field of the Extension
   * @returns The valueDuration-field
   */
  public getValueDuration(): Duration | undefined {
    return this.element.valueDuration
  }

  /**
   * Sets the valueDuration-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Duration } valueDuration - The value to be set in the valueDuration-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueDuration(valueDuration?: Duration): this {
    valueDuration
      ? (this.element.valueDuration = valueDuration)
      : delete this.element.valueDuration
    return this
  }

  /**
   * Returns the valueHumanName-field of the Extension
   * @returns The valueHumanName-field
   */
  public getValueHumanName(): HumanName | undefined {
    return this.element.valueHumanName
  }

  /**
   * Sets the valueHumanName-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { HumanName } valueHumanName - The value to be set in the valueHumanName-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueHumanName(valueHumanName?: HumanName): this {
    valueHumanName
      ? (this.element.valueHumanName = valueHumanName)
      : delete this.element.valueHumanName
    return this
  }

  /**
   * Returns the valueIdentifier-field of the Extension
   * @returns The valueIdentifier-field
   */
  public getValueIdentifier(): Identifier | undefined {
    return this.element.valueIdentifier
  }

  /**
   * Sets the valueIdentifier-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Identifier } valueIdentifier - The value to be set in the valueIdentifier-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueIdentifier(valueIdentifier?: Identifier): this {
    valueIdentifier
      ? (this.element.valueIdentifier = valueIdentifier)
      : delete this.element.valueIdentifier
    return this
  }

  /**
   * Returns the valueMoney-field of the Extension
   * @returns The valueMoney-field
   */
  public getValueMoney(): Money | undefined {
    return this.element.valueMoney
  }

  /**
   * Sets the valueMoney-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Money } valueMoney - The value to be set in the valueMoney-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueMoney(valueMoney?: Money): this {
    valueMoney
      ? (this.element.valueMoney = valueMoney)
      : delete this.element.valueMoney
    return this
  }

  /**
   * Returns the valuePeriod-field of the Extension
   * @returns The valuePeriod-field
   */
  public getValuePeriod(): Period | undefined {
    return this.element.valuePeriod
  }

  /**
   * Sets the valuePeriod-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Period } valuePeriod - The value to be set in the valuePeriod-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValuePeriod(valuePeriod?: Period): this {
    valuePeriod
      ? (this.element.valuePeriod = valuePeriod)
      : delete this.element.valuePeriod
    return this
  }

  /**
   * Returns the valueQuantity-field of the Extension
   * @returns The valueQuantity-field
   */
  public getValueQuantity(): Quantity | undefined {
    return this.element.valueQuantity
  }

  /**
   * Sets the valueQuantity-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Quantity } valueQuantity - The value to be set in the valueQuantity-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueQuantity(valueQuantity?: Quantity): this {
    valueQuantity
      ? (this.element.valueQuantity = valueQuantity)
      : delete this.element.valueQuantity
    return this
  }

  /**
   * Returns the valueRange-field of the Extension
   * @returns The valueRange-field
   */
  public getValueRange(): Range | undefined {
    return this.element.valueRange
  }

  /**
   * Sets the valueRange-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Range } valueRange - The value to be set in the valueRange-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueRange(valueRange?: Range): this {
    valueRange
      ? (this.element.valueRange = valueRange)
      : delete this.element.valueRange
    return this
  }

  /**
   * Returns the valueRatio-field of the Extension
   * @returns The valueRatio-field
   */
  public getValueRatio(): Ratio | undefined {
    return this.element.valueRatio
  }

  /**
   * Sets the valueRatio-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Ratio } valueRatio - The value to be set in the valueRatio-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueRatio(valueRatio?: Ratio): this {
    valueRatio
      ? (this.element.valueRatio = valueRatio)
      : delete this.element.valueRatio
    return this
  }

  /**
   * Returns the valueReference-field of the CarePlanActivity if it is set
   * @returns { Reference } The valueReference-field
   */
  public getValueReference(): Reference | undefined {
    return this.element.valueReference
  }

  /**
   * Sets the valueReference-field of the CarePlanActivity with the provided reference
   * If no argument is provided, the field will be removed
   * @returns { CarePlanActivityBuilder } The builder
   */
  public setValueReference(valueReference?: Reference): this {
    valueReference
      ? (this.element.valueReference = valueReference)
      : delete this.element.valueReference
    return this
  }

  /**
   * Returns the valueSampledData-field of the Extension
   * @returns The valueSampledData-field
   */
  public getValueSampledData(): SampledData | undefined {
    return this.element.valueSampledData
  }

  /**
   * Sets the valueSampledData-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { SampledData } valueSampledData - The value to be set in the valueSampledData-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueSampledData(valueSampledData?: SampledData): this {
    valueSampledData
      ? (this.element.valueSampledData = valueSampledData)
      : delete this.element.valueSampledData
    return this
  }

  /**
   * Returns the valueSignature-field of the Extension
   * @returns The valueSignature-field
   */
  public getValueSignature(): Signature | undefined {
    return this.element.valueSignature
  }

  /**
   * Sets the valueSignature-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Signature } valueSignature - The value to be set in the valueSignature-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueSignature(valueSignature?: Signature): this {
    valueSignature
      ? (this.element.valueSignature = valueSignature)
      : delete this.element.valueSignature
    return this
  }

  /**
   * Returns the valueTiming-field of the Extension
   * @returns The valueTiming-field
   */
  public getValueTiming(): Timing | undefined {
    return this.element.valueTiming
  }

  /**
   * Sets the valueTiming-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Timing } valueTiming - The value to be set in the valueTiming-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueTiming(valueTiming?: Timing): this {
    valueTiming
      ? (this.element.valueTiming = valueTiming)
      : delete this.element.valueTiming
    return this
  }

  /**
   * Returns the valueMeta-field of the Extension
   * @returns The valueMeta-field
   */
  public getValueMeta(): Meta | undefined {
    return this.element.valueMeta
  }

  /**
   * Sets the valueMeta-field of the Extension
   * If no argument is provided, the field will be removed
   * @param { Meta } valueMeta - The value to be set in the valueMeta-field
   * @returns { ExtensionBuilder } - The builder
   */
  public setValueMeta(valueMeta?: Meta): this {
    valueMeta
      ? (this.element.valueMeta = valueMeta)
      : delete this.element.valueMeta
    return this
  }

  /**
   * Make a new object that implements a FHIR Extension
   * @returns { Extension } - an object that implements FHIR Extension
   * @override
   */
  protected make(): Extension {
    return {} as Extension
  }

  /**
   * Returns the extension
   * @returns { Extension } The extension
   * @override
   */
  public build(): Extension {
    return this.element
  }
}
