import {
  PlanDefinition,
  PlanDefinitionAction,
  id,
  uri,
  canonical,
  Basic,
} from 'fhir'
import { DomainResourceBuilder } from './domain-resource-builder'
import { PlanDefinitionActionBuilder } from './plan-definition-action-builder'
import { AlreadyExistsError } from './exceptions'
import { _generateIsoDate } from './utils/date-util'
import { ThresholdSetBuilder } from './threshold-set-builder'

/**
 * Class representing a FHIR PlanDefinition builder
 * FIXME: Incomplete
 */
export class PlanDefinitionBuilder extends DomainResourceBuilder {
  /**
   * The PlanDefinition that is being built
   * @protected
   * @type { PlanDefinition }
   * @ignore
   */
  protected resource: PlanDefinition

  /**
   * Creates a new FHIR PlanDefinitionBuilder
   */
  public constructor() {
    super()
    this.resource = this.make()
  }

  /**
   * Sets a copy of an existing plandefinition for this PlanDefinitionBuilder to allow for manipulating, or interacting with, the
   * plandefinition through this PlanDefinitionBuilder
   * If no argument is provided, a new PlanDefinition will be generated
   * @param { PlanDefinition } plandefinition - The plandefinition to set
   * @returns { PlanDefinitionBuilder } This PlanDefinitionBuilder
   * @override
   */
  public use(planDefinition?: PlanDefinition): this {
    this.resource = planDefinition ? { ...planDefinition } : this.make()
    return this
  }

  /**
   * Checks if a Questionnaire with a specific canonical is in the PlanDefinition.
   * @param canonical The canonical of the Questionnaire to check for
   * @returns { boolean } A boolean value indicating whether the Questionnaire is there or not.
   */
  public has(canonical: canonical): boolean {
    // check if an PlanDefinitionAction has the same canonical reference
    const action = this.getAction().find(
      action => action.definitionCanonical === canonical
    )
    if (action) return true

    // check if a ThresholdSet has the same canonical reference
    const thresholdSet = this.getContained().find(contained => {
      if (contained.resourceType !== 'Basic') return false
      const tsBuilder = new ThresholdSetBuilder().use(contained as Basic)
      if (tsBuilder.getAppliesTo() === canonical) return true
      return false
    })
    if (thresholdSet) return true
    return false
  }

  /**
   * Returns the url-field of the PlanDefinition
   * @returns { uri } The url-field
   */
  public getUrl(): uri | undefined {
    return this.resource.url
  }

  /**
   * Sets the url-field of the PlanDefinition
   * If no argument is provided, the field will be removed
   * @param { uri } url - The value to be set in the url-field
   * @returns { PlanDefinitionBuilder } - This PlanDefinitionBuilder
   */
  public setUrl(url?: uri): this {
    url ? (this.resource.url = url) : delete this.resource.url
    return this
  }

  /**
   * Returns the version-field of the PlanDefinition
   * @returns { string } The version-field
   */
  public getVersion(): string | undefined {
    return this.resource.version
  }

  /**
   * Sets the version-field of the Coding
   * If no argument is provided, the field will be removed
   * @param { string } version - The value to be set in the version-field
   * @returns { PlanDefinitionBuilder } - This PlanDefinitionBuilder
   */
  public setVersion(version?: string): this {
    version ? (this.resource.version = version) : delete this.resource.version
    return this
  }

  /**
   * Returns either the planDefintitionAction identified by the provided id from the action-array or the entire action-array, if no id is provided
   * @returns { PlanDefinitionAction[] } The planDefintitionAction identified by the provided id or the action-array
   */
  public getAction(): PlanDefinitionAction[]
  public getAction(id: id): PlanDefinitionAction | undefined
  public getAction(
    id?: id
  ): PlanDefinitionAction | undefined | PlanDefinitionAction[] {
    if (id) {
      return this.getAction().find(action => action.id === id)
    }
    return this.resource.action || []
  }

  /**
   * Sets the action-array to the provided array
   * If no argument is provided, the field will be removed
   * @param { PlanDefinitionAction[] } action The array to be set as the action-array
   * @returns the Builder
   */
  public setAction(action?: PlanDefinitionAction[]): this {
    action ? (this.resource.action = action) : delete this.resource.action
    return this
  }

  /**
   * Adds the action provided as an argument to the resource's action-array
   * @param { PlanDefinitionAction } action The action to be added to the action-array
   * @returns the Builder
   */
  public addAction(action: PlanDefinitionAction): this {
    if (this.getAction().length > 0) {
      if (
        this.getAction().find(act =>
          new PlanDefinitionActionBuilder().use(act).conflictsWith(action)
        )
      ) {
        throw new AlreadyExistsError(
          'The planDefinitionAction is in conflict with one of the existing references in the action-array'
        )
      }
    }
    this.setAction([...this.getAction(), action])
    return this
  }

  /**
   * Creates an ISO-string for current time
   * @returns { string } An ISO-string for current time
   * @ignore
   */
  /* istanbul ignore next */
  public _generateIsoDate(): string {
    return _generateIsoDate()
  }

  /**
   * Make a new object that implements a FHIR PlanDefinition
   * @returns { PlanDefinition } - an object that implements FHIR PlanDefinition
   * @override
   */
  protected make(): PlanDefinition {
    return {
      resourceType: 'PlanDefinition',
      date: this._generateIsoDate(),
    } as PlanDefinition
  }

  /**
   * Builds and returns the plandefinition
   * @returns { PlanDefinition } - the plandefinition
   * @override
   */
  public build(): PlanDefinition {
    return this.resource
  }
}
