import {
  uri,
  Task,
  Identifier,
  Reference,
  TaskStatus,
  TaskIntent,
  dateTime,
  TaskInput,
} from 'fhir'
import { AlreadyExistsError, MissingPropertyError } from './exceptions'
import { DomainResourceBuilder } from './domain-resource-builder'
import { ReferenceBuilder } from './reference-builder'
import { TaskInputBuilder } from './task-input-builder'

/**
 * Class reporesenting a FHIR Task builder
 * FIXME: Incomplete
 */
export class TaskBuilder extends DomainResourceBuilder {
  /**
   * The Task that is being built
   * @protected
   * @type { Task }
   * @ignore
   */
  protected resource: Task

  /**
   * Create a new FHIR TaskBuilder
   */
  public constructor() {
    super()
    this.resource = this.make()
  }

  /**
   * Sets a copy of an existing task for this TaskBuilder to allow for manipulation,
   * or interacting with, the task through this TaskBuilder.
   * If no argument is provided, a new Task will be generated
   * @param { Task } task - the task to be set
   * @returns { TaskBuilder } this TaskBuilder
   * @override
   */
  public use(task?: Task): this {
    this.resource = task ? { ...task } : this.make()
    return this
  }

  /**
   * Returns identifier from specific system if one exists or the identifier-array if no system was provided
   * @param { uri } system - the system from which to return the identifier
   * @returns { Identifier } the identifier if it exists
   */
  public getIdentifier(): Identifier[]
  public getIdentifier(system: uri): Identifier | undefined
  public getIdentifier(system?: uri): Identifier[] | Identifier | undefined {
    if (!system) {
      return this.resource.identifier || []
    }
    return this.getIdentifier().find(identifier => identifier.system === system)
  }

  /**
   * Replaces the identifier array with the provided array
   * If no argument is provided, the field will be removed
   * @param { Identifier[] } identifiers - the identifier to be set
   * @returns { TaskBuilder } - this TaskBuilder
   */
  public setIdentifier(identifier?: Identifier[]): this {
    identifier
      ? (this.resource.identifier = identifier)
      : delete this.resource.identifier
    return this
  }

  /**
   * Appends an identifier to the task, if an identifier within the system does not already exist
   * @param { Identifier } identifier - the identifier to be appended
   * @returns { TaskBuilder } - this TaskBuilder
   * @throws { MissingPropertyError } When the identifier is missing necessary properties
   * @throws { AlreadyExistsError } When an identifier in that system already exists on the task
   */
  public addIdentifier(identifier: Identifier): this {
    if (!identifier.system || !identifier.value) {
      throw new MissingPropertyError(
        'Identifiers must have both a system and a value.'
      )
    }
    if (
      this.getIdentifier().find(ident => ident.system === identifier.system)
    ) {
      throw new AlreadyExistsError(
        'An identifier with that system, already exists in the identifier-array'
      )
    }
    this.setIdentifier([...this.getIdentifier(), identifier])
    return this
  }

  /**
   * Returns the basedOn-array
   * @returns { Reference[] } The basedOn-array
   */
  public getBasedOn(): Reference[] {
    return this.resource.basedOn || []
  }

  /**
   * Sets the basedOn-array to the provided array
   * If no argument is provided, the field will be removed
   * @param { Reference[] } basedOn The array to be set as the basedOn-array
   * @returns the Builder
   */
  public setBasedOn(basedOn?: Reference[]): this {
    if (!basedOn) delete this.resource.basedOn
    else {
      basedOn.forEach(reference => {
        if (
          !reference.reference &&
          (!reference.identifier ||
            (!reference.identifier.system || !reference.identifier.value))
        ) {
          throw new MissingPropertyError(
            'All references must have either a literal reference or an identifier with both system and value.'
          )
        }
      })
      this.resource.basedOn = basedOn
    }
    return this
  }

  /**
   * Adds the reference provided as an argument to the resource's reference array, if a reference referring to the same resource does not already exist in the array
   * @param { Reference } reference The reference to be added to the basedOn-array
   * @returns the Builder
   */
  public addBasedOn(reference: Reference): this {
    if (this.getBasedOn().length > 0) {
      if (
        this.getBasedOn().find(ref =>
          new ReferenceBuilder().use(ref).conflictsWith(reference)
        )
      ) {
        throw new AlreadyExistsError(
          'The reference is in conflict with one of the existing references in the basedOn-array'
        )
      }
    }
    this.setBasedOn([...this.getBasedOn(), reference])
    return this
  }

  /**
   * Returns the status field of the task
   * @returns { code } the code of the task
   */
  public getStatus(): TaskStatus {
    return this.resource.status
  }

  /**
   * Set the status field of the task
   * If no argument is provided, the field will be removed
   * @param { code } status - the status to be set
   */
  public setStatus(status: TaskStatus): this {
    this.resource.status = status
    return this
  }

  /**
   * Returns the intent field of the task
   * @returns { TaskIntent } the intent of the task
   */
  public getIntent(): TaskIntent {
    return this.resource.intent
  }

  /**
   * Set the intent field of the task
   * If no argument is provided, the field will be removed
   * @param { TaskIntent } intent - the intent to be set.
   *              Must be one of: unknown | proposal | plan | order | original-order | reflex-order | filler-order
   * @returns { TaskBuilder } this TaskBuilder
   */
  public setIntent(intent: TaskIntent): this {
    this.resource.intent = intent
    return this
  }

  /**
   * Returns the authoredOn field of the task if it is set
   * @returns { dateTime } the authoredOn of the task
   */
  public getAuthoredOn(): dateTime | undefined {
    return this.resource.authoredOn
  }

  /**
   * Set the authoredOn field of the task
   * If no argument is provided, the field will be removed
   * @param { dateTime } authoredOn - the date to be set as authoredOn
   * @returns { TaskBuilder } this TaskBuilder
   */
  public setAuthoredOn(authoredOn?: dateTime): this {
    authoredOn
      ? (this.resource.authoredOn = authoredOn)
      : delete this.resource.authoredOn
    return this
  }

  /**
   * Returns the lastModified field of the task if it is set
   * @returns { dateTime } the lastModified of the task
   */
  public getLastModified(): dateTime | undefined {
    return this.resource.lastModified
  }

  /**
   * Set the lastModified field of the task
   * If no argument is provided, the field will be removed
   * @param { dateTime } lastModified - the date to be set as lastModified
   * @returns { TaskBuilder } this TaskBuilder
   */
  public setLastModified(lastModified?: dateTime): this {
    lastModified
      ? (this.resource.lastModified = lastModified)
      : delete this.resource.lastModified
    return this
  }

  /**
   * Returns the input-array
   * @returns { TaskInput[] } The input-array
   */
  public getInput(): TaskInput[] {
    return this.resource.input || []
  }

  /**
   * Sets the input-array to the provided array
   * If no argument is provided, the field will be removed
   * @param { TaskInput[] } input The array to be set as the input-array
   * @returns the Builder
   */
  public setInput(input?: TaskInput[]): this {
    input ? (this.resource.input = input) : delete this.resource.input
    return this
  }

  /**
   * Adds the reference provided as an argument to the resource's reference array, if a reference referring to the same resource does not already exist in the array
   * @param { TaskInput } reference The reference to be added to the input-array
   * @returns the Builder
   */
  public addInput(input: TaskInput): this {
    if (
      this.getInput().find(taskInput =>
        new TaskInputBuilder().use(taskInput).conflictsWith(input)
      )
    ) {
      throw new AlreadyExistsError(
        'The taskInput is in conflict with one of the existing inputs in the input-array'
      )
    }
    this.setInput([...this.getInput(), input])
    return this
  }

  /**
   * Returns whether or not the task provided as an argument is in conflict with the task used in the builder
   * @param { Task } task The task to test for conflict
   * @returns { boolean } Whether it is in conflict
   */
  public conflictsWith(task: Task) {
    let conflict: boolean = super.conflictsWith(task)
    const taskBuilder: TaskBuilder = new TaskBuilder().use(task)

    this.getInput().forEach(input => {
      taskBuilder.getInput().forEach(otherInput => {
        conflict = new TaskInputBuilder().use(input).conflictsWith(otherInput)
      })
    })
    return conflict
  }

  /**
   * Make a new object that implements a FHIR Task
   * @returns { Task } - an object that implements FHIR Task
   * @override
   */
  protected make(): Task {
    const task = {
      resourceType: 'Task',
    } as Task
    return task
  }

  /**
   * Builds and returns the task
   * @returns { Task } - the task
   * @override
   */
  public build(): Task {
    return this.resource
  }
}
