import { TaskInput, CodeableConcept, canonical, Reference } from 'fhir'
import { BackboneElementBuilder } from './backbone-element-builder'
import { ReferenceBuilder } from './reference-builder'

/** A class representing a FHIR TaskInputBuilder */
export class TaskInputBuilder extends BackboneElementBuilder {
  /**
   * The TaskInput being built
   * @private
   * @type { TaskInput }
   * @ignore
   */
  protected element: TaskInput

  /**
   * Creates a new TaskInputBuilder
   */
  public constructor() {
    super()
    this.element = this.make()
  }

  /**
   * Sets a copy of an existing taskInput for this builder to allow for manipulation
   * If no argument is provided, a new TaskInput will be generated
   * @param { TaskInput } taskInput - the taskInput to be set
   * @returns { TaskInputBuilder } The builder
   * @override
   */
  public use(taskInput?: TaskInput): this {
    this.element = taskInput ? { ...taskInput } : this.make()
    return this
  }

  /**
   * Returns the type-field of the TaskInput
   * @returns { CodeableConcept } The type-field
   */
  public getType(): CodeableConcept | undefined {
    return this.element.type
  }

  /**
   * Sets the type-field of the TaskInput
   * If no argument is provided, the field will be removed
   * @param { CodeableConcept } type - The value to be set in the type-field
   * @returns { TaskInputBuilder } - This TaskInputBuilder
   */
  public setType(type?: CodeableConcept): this {
    type ? (this.element.type = type) : delete this.element.type
    return this
  }

  /**
   * Returns the valueCanonical-field of the TaskInput
   * @returns { canonical } The valueCanonical-field
   */
  public getValueCanonical(): canonical | undefined {
    return this.element.valueCanonical
  }

  /**
   * Sets the valueCanonical-field of the TaskInput
   * If no argument is provided, the field will be removed
   * @param { canonical } valueCanonical - The value to be set in the valueCanonical-field
   * @returns { TaskInputBuilder } - This TaskInputBuilder
   */
  public setValueCanonical(valueCanonical?: canonical): this {
    valueCanonical
      ? (this.element.valueCanonical = valueCanonical)
      : delete this.element.valueCanonical
    return this
  }

  /**
   * Returns the valueReference-field of the TaskInput
   * @returns { Reference } The valueReference-field
   */
  public getValueReference(): Reference | undefined {
    return this.element.valueReference
  }

  /**
   * Sets the valueReference-field of the TaskInput
   * If no argument is provided, the field will be removed
   * @param { Reference } valueReference - The value to be set in the valueReference-field
   * @returns { TaskInputBuilder } - This TaskInputBuilder
   */
  public setValueReference(valueReference?: Reference): this {
    valueReference
      ? (this.element.valueReference = valueReference)
      : delete this.element.valueReference
    return this
  }

  /**
   * Returns whether or not the taskInput provided as an argument is in conflict with the taskInput used in the builder
   * @param { TaskInput } taskInput The taskInput to test for conflict
   * @returns { boolean } Whether it is in conflict
   * @override
   */
  public conflictsWith(taskInput: TaskInput) {
    let conflict = super.conflictsWith(taskInput)
    const inputBuilder: TaskInputBuilder = new TaskInputBuilder().use(taskInput)

    if (
      this.getValueCanonical() &&
      this.getValueCanonical() === inputBuilder.getValueCanonical()
    ) {
      conflict = true
    }

    if (
      this.getValueReference() &&
      inputBuilder.getValueReference() &&
      new ReferenceBuilder()
        .use(this.getValueReference())
        .conflictsWith(inputBuilder.getValueReference() as Reference)
    ) {
      conflict = true
    }
    return conflict
  }

  /**
   * Creates a new TaskInput
   * @returns { TaskInput } The taskInput
   * @override
   */
  protected make(): TaskInput {
    return {} as TaskInput
  }

  /**
   * Builds and returns the TaskInput
   * @returns { TaskInput } - the TaskInput
   * @override
   */
  public build(): TaskInput {
    return this.element
  }
}
