import {
  Extension,
  Coding,
  decimal,
  integer,
  date,
  dateTime,
  uri,
  Attachment,
  CodeableConcept,
  Quantity,
  Reference,
  time,
} from 'fhir'
import { Constants } from './global/constants'
import { MalformedError } from './exceptions/malformed-error'
import { ElementBuilder } from './element-builder'
import { ExtensionBuilder } from './extension-builder'

/**
 * Class representing an EffectiveWhen builder for ThresholdSet Thresholds
 */
export class ThresholdEffectiveWhenBuilder extends ElementBuilder {
  /**
   * The effectiveWhen that is being built
   * @protected
   * @type { Extension }
   * @ignore
   */
  protected element: Extension

  /**
   * Creates a new EffectiveWhen for FHIR ThresholdSet Threholds
   */
  public constructor() {
    super()
    this.element = this.make()
  }

  /**
   * Sets a copy of an existing effectiveWhen for this builder to allow for manipulating, or interacting with, the
   * effectiveWhen through this builder
   * If no argument is provided, a new ThresholdEffectiveWhen will be generated
   * @param { Extension } threshold - The effectiveWhen to set
   * @returns { ThresholdEffectiveWhenBuilder } The builder
   * @override
   */
  public use(effectiveWhen?: Extension): this {
    this.element = effectiveWhen ? { ...effectiveWhen } : this.make()
    return this
  }

  /**
   * Returns the component on which this effectiveWhen is acting
   * @returns { Coding } The component
   */
  public getComponent(): Coding | undefined {
    const extension = this.getExtension(Constants.thresholdSet.component)
    if (extension.length < 1) return
    if (extension.length > 1 || !extension[0].valueCoding) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed component-extension'
      )
    }
    return extension[0].valueCoding
  }

  /**
   * Sets the component on which this effectiveWhen is acting
   * If no argument is provided, the field will be removed
   * @param { Coding } component - The component
   * @returns { ThresholdEffectiveWhenBuilder } - The builder
   */
  public setComponent(component?: Coding): this {
    this.setExtension([
      ...this.getExtension().filter(
        extension => extension.url !== Constants.thresholdSet.component
      ),
    ])
    if (component) {
      this.addExtension(
        new ExtensionBuilder()
          .setUrl(Constants.thresholdSet.component)
          .setValueCoding(component)
          .build()
      )
    }
    return this
  }

  /**
   * Returns the valueBoolean of the effectiveWhen
   * @returns { boolean } The valueBoolean
   */
  public getValueBoolean(): boolean | undefined {
    const extension = this.getExtension(Constants.thresholdSet.valueBoolean)
    if (extension.length < 1) return
    if (extension.length > 1 || !extension[0].valueBoolean) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed valueBoolean-extension'
      )
    }
    return extension[0].valueBoolean
  }

  /**
   * Sets the valueBoolean of the effectiveWhen
   * If no argument is provided, the field will be removed
   * @param { boolean } valueBoolean - The valueBoolean
   * @returns { ThresholdEffectiveWhenBuilder } - The builder
   */
  public setValueBoolean(bool?: boolean): this {
    this.setExtension([
      ...this.getExtension().filter(
        extension => extension.url !== Constants.thresholdSet.valueBoolean
      ),
    ])
    if (typeof bool !== 'undefined') {
      this.addExtension(
        new ExtensionBuilder()
          .setUrl(Constants.thresholdSet.valueBoolean)
          .setValueBoolean(bool)
          .build()
      )
    }
    return this
  }

  /**
   * Returns the valueDecimal of the effectiveWhen
   * @returns { decimal } The valueDecimal
   */
  public getValueDecimal(): decimal | undefined {
    const extension = this.getExtension(Constants.thresholdSet.valueDecimal)
    if (extension.length < 1) return
    if (extension.length > 1 || !extension[0].valueDecimal) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed valueDecimal-extension'
      )
    }
    return extension[0].valueDecimal
  }

  /**
   * Sets the valueDecimal of the effectiveWhen
   * If no argument is provided, the field will be removed
   * @param { decimal } valueDecimal - The valueDecimal
   * @returns { ThresholdEffectiveWhenBuilder } - The builder
   */
  public setValueDecimal(decimal?: decimal): this {
    this.setExtension([
      ...this.getExtension().filter(
        extension => extension.url !== Constants.thresholdSet.valueDecimal
      ),
    ])
    if (decimal) {
      this.addExtension(
        new ExtensionBuilder()
          .setUrl(Constants.thresholdSet.valueDecimal)
          .setValueDecimal(decimal)
          .build()
      )
    }
    return this
  }

  /**
   * Returns the valueInteger of the effectiveWhen
   * @returns { integer } The valueInteger
   */
  public getValueInteger(): integer | undefined {
    const extension = this.getExtension(Constants.thresholdSet.valueInteger)
    if (extension.length < 1) return
    if (extension.length > 1 || !extension[0].valueInteger) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed valueInteger-extension'
      )
    }
    return extension[0].valueInteger
  }

  /**
   * Sets the valueInteger of the effectiveWhen
   * If no argument is provided, the field will be removed
   * @param { integer } valueInteger - The valueInteger
   * @returns { ThresholdEffectiveWhenBuilder } - The builder
   */
  public setValueInteger(integer?: integer): this {
    this.setExtension([
      ...this.getExtension().filter(
        extension => extension.url !== Constants.thresholdSet.valueInteger
      ),
    ])
    if (integer) {
      this.addExtension(
        new ExtensionBuilder()
          .setUrl(Constants.thresholdSet.valueInteger)
          .setValueInteger(integer)
          .build()
      )
    }
    return this
  }

  /**
   * Returns the valueString of the effectiveWhen
   * @returns { string } The valueString
   */
  public getValueString(): string | undefined {
    const extension = this.getExtension(Constants.thresholdSet.valueString)
    if (extension.length < 1) return
    if (extension.length > 1 || !extension[0].valueString) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed valueString-extension'
      )
    }
    return extension[0].valueString
  }

  /**
   * Sets the valueString of the effectiveWhen
   * If no argument is provided, the field will be removed
   * @param { string } valueString - The valueString
   * @returns { ThresholdEffectiveWhenBuilder } - The builder
   */
  public setValueString(str?: string): this {
    this.setExtension([
      ...this.getExtension().filter(
        extension => extension.url !== Constants.thresholdSet.valueString
      ),
    ])
    if (str) {
      this.addExtension(
        new ExtensionBuilder()
          .setUrl(Constants.thresholdSet.valueString)
          .setValueString(str)
          .build()
      )
    }
    return this
  }

  /**
   * Returns the valueDate of the effectiveWhen
   * @returns { date } The valueDate
   */
  public getValueDate(): date | undefined {
    const extension = this.getExtension(Constants.thresholdSet.valueDate)
    if (extension.length < 1) return
    if (extension.length > 1 || !extension[0].valueDate) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed valueDate-extension'
      )
    }
    return extension[0].valueDate
  }

  /**
   * Sets the valueDate of the effectiveWhen
   * If no argument is provided, the field will be removed
   * @param { date } valueDate - The valueDate
   * @returns { ThresholdEffectiveWhenBuilder } - The builder
   */
  public setValueDate(date?: date): this {
    this.setExtension([
      ...this.getExtension().filter(
        extension => extension.url !== Constants.thresholdSet.valueDate
      ),
    ])
    if (date) {
      this.addExtension(
        new ExtensionBuilder()
          .setUrl(Constants.thresholdSet.valueDate)
          .setValueDate(date)
          .build()
      )
    }
    return this
  }

  /**
   * Returns the valueDateTime of the effectiveWhen
   * @returns { dateTime } The valueDateTime
   */
  public getValueDateTime(): dateTime | undefined {
    const extension = this.getExtension(Constants.thresholdSet.valueDateTime)
    if (extension.length < 1) return
    if (extension.length > 1 || !extension[0].valueDateTime) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed valueDateTime-extension'
      )
    }
    return extension[0].valueDateTime
  }

  /**
   * Sets the valueDateTime of the effectiveWhen
   * If no argument is provided, the field will be removed
   * @param { time } valueTime - The valueTime
   * @returns { ThresholdEffectiveWhenBuilder } - The builder
   */
  public setValueDateTime(dateTime?: dateTime): this {
    this.setExtension([
      ...this.getExtension().filter(
        extension => extension.url !== Constants.thresholdSet.valueDateTime
      ),
    ])
    if (dateTime) {
      this.addExtension(
        new ExtensionBuilder()
          .setUrl(Constants.thresholdSet.valueDateTime)
          .setValueDateTime(dateTime)
          .build()
      )
    }
    return this
  }

  /**
   * Returns the valueTime of the effectiveWhen
   * @returns { time } The valueTime
   */
  public getValueTime(): time | undefined {
    const extension = this.getExtension(Constants.thresholdSet.valueTime)
    if (extension.length < 1) return
    if (extension.length > 1 || !extension[0].valueTime) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed valueTime-extension'
      )
    }
    return extension[0].valueTime
  }

  /**
   * Sets the valueTime of the effectiveWhen
   * If no argument is provided, the field will be removed
   * @param { time } valueTime - The valueDateTime
   * @returns { ThresholdEffectiveWhenBuilder } - The builder
   */
  public setValueTime(time?: time): this {
    this.setExtension([
      ...this.getExtension().filter(
        extension => extension.url !== Constants.thresholdSet.valueTime
      ),
    ])
    if (time) {
      this.addExtension(
        new ExtensionBuilder()
          .setUrl(Constants.thresholdSet.valueTime)
          .setValueTime(time)
          .build()
      )
    }
    return this
  }

  /**
   * Returns the valueUri of the effectiveWhen
   * @returns { uri } The valueUri
   */
  public getValueUri(): uri | undefined {
    const extension = this.getExtension(Constants.thresholdSet.valueUri)
    if (extension.length < 1) return
    if (extension.length > 1 || !extension[0].valueUri) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed valueUri-extension'
      )
    }
    return extension[0].valueUri
  }

  /**
   * Sets the valueUri of the effectiveWhen
   * If no argument is provided, the field will be removed
   * @param { uri } valueUri - The valueUri
   * @returns { ThresholdEffectiveWhenBuilder } - The builder
   */
  public setValueUri(uri?: uri): this {
    this.setExtension([
      ...this.getExtension().filter(
        extension => extension.url !== Constants.thresholdSet.valueUri
      ),
    ])
    if (uri) {
      this.addExtension(
        new ExtensionBuilder()
          .setUrl(Constants.thresholdSet.valueUri)
          .setValueUri(uri)
          .build()
      )
    }
    return this
  }

  /**
   * Returns the valueAttachment of the effectiveWhen
   * @returns { Attachment } The valueAttachment
   */
  public getValueAttachment(): Attachment | undefined {
    const extension = this.getExtension(Constants.thresholdSet.valueAttachment)
    if (extension.length < 1) return
    if (extension.length > 1 || !extension[0].valueAttachment) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed valueAttachment-extension'
      )
    }
    return extension[0].valueAttachment
  }

  /**
   * Sets the valueAttachment of the effectiveWhen
   * If no argument is provided, the field will be removed
   * @param { Attachment } valueAttachment - The valueAttachment
   * @returns { ThresholdEffectiveWhenBuilder } - The builder
   */
  public setValueAttachment(attachment?: Attachment): this {
    this.setExtension([
      ...this.getExtension().filter(
        extension => extension.url !== Constants.thresholdSet.valueAttachment
      ),
    ])
    if (attachment) {
      this.addExtension(
        new ExtensionBuilder()
          .setUrl(Constants.thresholdSet.valueAttachment)
          .setValueAttachment(attachment)
          .build()
      )
    }
    return this
  }

  /**
   * Returns the valueCodeableConcept of the effectiveWhen
   * @returns { CodeableConcept } The valueCodeableConcept
   */
  public getValueCodeableConcept(): CodeableConcept | undefined {
    const extension = this.getExtension(
      Constants.thresholdSet.valueCodeableConcept
    )
    if (extension.length < 1) return
    if (extension.length > 1 || !extension[0].valueCodeableConcept) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed valueCodeableConcept-extension'
      )
    }
    return extension[0].valueCodeableConcept
  }

  /**
   * Sets the valueCodeableConcept of the effectiveWhen
   * If no argument is provided, the field will be removed
   * @param { CodeableConcept } valueCodeableConcept - The valueCodeableConcept
   * @returns { ThresholdEffectiveWhenBuilder } - The builder
   */
  public setValueCodeableConcept(codeableConcept?: CodeableConcept): this {
    this.setExtension([
      ...this.getExtension().filter(
        extension =>
          extension.url !== Constants.thresholdSet.valueCodeableConcept
      ),
    ])
    if (codeableConcept) {
      this.addExtension(
        new ExtensionBuilder()
          .setUrl(Constants.thresholdSet.valueCodeableConcept)
          .setValueCodeableConcept(codeableConcept)
          .build()
      )
    }
    return this
  }

  /**
   * Returns the valueQuantity of the effectiveWhen
   * @returns { Quantity } The valueQuantity
   */
  public getValueQuantity(): Quantity | undefined {
    const extension = this.getExtension(Constants.thresholdSet.valueQuantity)
    if (extension.length < 1) return
    if (extension.length > 1 || !extension[0].valueQuantity) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed valueQuantity-extension'
      )
    }
    return extension[0].valueQuantity
  }

  /**
   * Sets the valueQuantity of the effectiveWhen
   * If no argument is provided, the field will be removed
   * @param { Quantity } valueQuantity - The valueQuantity
   * @returns { ThresholdEffectiveWhenBuilder } - The builder
   */
  public setValueQuantity(quantity?: Quantity): this {
    this.setExtension([
      ...this.getExtension().filter(
        extension => extension.url !== Constants.thresholdSet.valueQuantity
      ),
    ])
    if (quantity) {
      this.addExtension(
        new ExtensionBuilder()
          .setUrl(Constants.thresholdSet.valueQuantity)
          .setValueQuantity(quantity)
          .build()
      )
    }
    return this
  }

  /**
   * Returns the valueReference of the effectiveWhen
   * @returns { Reference } The valueReference
   */
  public getValueReference(): Reference | undefined {
    const extension = this.getExtension(Constants.thresholdSet.valueReference)
    if (extension.length < 1) return
    if (extension.length > 1 || !extension[0].valueReference) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed valueReference-extension'
      )
    }
    return extension[0].valueReference
  }

  /**
   * Sets the valueReference of the effectiveWhen
   * If no argument is provided, the field will be removed
   * @param { Reference } valueReference - The valueReference
   * @returns { ThresholdEffectiveWhenBuilder } - The builder
   */
  public setValueReference(reference?: Reference): this {
    this.setExtension([
      ...this.getExtension().filter(
        extension => extension.url !== Constants.thresholdSet.valueReference
      ),
    ])
    if (reference) {
      this.addExtension(
        new ExtensionBuilder()
          .setUrl(Constants.thresholdSet.valueReference)
          .setValueReference(reference)
          .build()
      )
    }
    return this
  }

  /**
   * Returns the operator of the effectiveWhen
   * @returns { EffectiveWhenOperator } The operator
   */
  public getOperator(): EffectiveWhenOperator | undefined {
    const extension = this.getExtension(Constants.thresholdSet.operator)
    if (extension.length < 1) return
    if (extension.length > 1 || !extension[0].valueCode) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed operator-extension'
      )
    }
    const operator = extension[0].valueCode
    if (
      operator !== 'exists' &&
      operator !== '=' &&
      operator !== '!=' &&
      operator !== '>' &&
      operator !== '<' &&
      operator !== '>=' &&
      operator !== '<='
    ) {
      throw new MalformedError(
        'The effectiveWhen contains a malformed operator-extension'
      )
    }
    return operator
  }

  /**
   * Sets the operator of the effectiveWhen
   * If no argument is provided, the field will be removed
   * @param { EffectiveWhenOperator } operator - The operator
   * @returns { ThresholdEffectiveWhenBuilder } - The builder
   */
  public setOperator(operator?: EffectiveWhenOperator): this {
    this.setExtension([
      ...this.getExtension().filter(
        extension => extension.url !== Constants.thresholdSet.operator
      ),
    ])
    if (operator) {
      this.addExtension(
        new ExtensionBuilder()
          .setUrl(Constants.thresholdSet.operator)
          .setValueCode(operator)
          .build()
      )
    }
    return this
  }

  /**
   * Make a new object that implements a FHIR ThresholdSet
   * @returns { ThresholdSet } - an object that implements FHIR ThresholdSet
   * @override
   */
  protected make(): Extension {
    return {
      url: Constants.thresholdSet.effectiveWhen,
      extension: [],
    }
  }

  /**
   * Builds and returns the plandefinition
   * @returns { Extension } - the plandefinition
   * @override
   */
  public build(): Extension {
    return this.element
  }
}

/**
 * All possible values of effectiveWhen.operator
 */
type EffectiveWhenOperator = 'exists' | '=' | '!=' | '>' | '<' | '>=' | '<='
