import { expect } from 'chai'
import { CarePlanActivity, CodeableConcept, Reference } from 'fhir'
import { CarePlanActivityBuilder } from '../src/care-plan-activity-builder'
import { ReferenceBuilder } from '../src/reference-builder'
import { CodeableConceptBuilder } from '../src/codeable-concept-builder'
import { ExtensionBuilder } from '../src/extension-builder'
import { Constants } from 'global/constants'

describe('CarePlanActivityBuilder', () => {
  it('should be able to build minimal CarePlanActivity', () => {
    const carePlanActivity: CarePlanActivity = new CarePlanActivityBuilder().build()
    expect(carePlanActivity).to.eql({})
  })

  it('should be able to accept existing CarePlanActivity', () => {
    const carePlanActivity: CarePlanActivity = {
      reference: {
        reference: '#task1',
        type: 'Task',
      },
    }
    expect(new CarePlanActivityBuilder().use(carePlanActivity).build()).to.eql({
      reference: {
        reference: '#task1',
        type: 'Task',
      },
    })
  })

  it('should be able to reset CarePlanActivity-element', () => {
    const carePlanActivity: CarePlanActivity = {
      reference: {
        reference: '#task1',
        type: 'Task',
      },
    }
    const carePlanActivityBuilder: CarePlanActivityBuilder = new CarePlanActivityBuilder().use(
      carePlanActivity
    )
    carePlanActivityBuilder.use()
    expect(carePlanActivityBuilder.build()).to.eql({})
  })

  it('should be able to set outcomeCodeableConcept-array', () => {
    const codeableConcept: CodeableConcept = new CodeableConceptBuilder()
      .setCoding([{ system: 'system', version: '2.0.0', code: 'code' }])
      .build()

    const carePlanActivity: CarePlanActivity = new CarePlanActivityBuilder()
      .setOutcomeCodeableConcept([codeableConcept])
      .build()

    expect(carePlanActivity).to.eql({
      outcomeCodeableConcept: [
        { coding: [{ system: 'system', version: '2.0.0', code: 'code' }] },
      ],
    })
  })

  it('should be able to unset outcomeCodeableConcept-array', () => {
    const carePlanActivityBuilder: CarePlanActivityBuilder = new CarePlanActivityBuilder().use(
      {
        outcomeCodeableConcept: [
          { coding: [{ system: 'system', version: '2.0.0', code: 'code' }] },
        ],
      }
    )

    expect(carePlanActivityBuilder.setOutcomeCodeableConcept().build()).to.eql(
      {}
    )
  })

  it('should be able to return outcomeCodeableConcept-array', () => {
    const carePlanActivity: CarePlanActivity = {
      outcomeCodeableConcept: [
        {
          coding: [{ system: 'system', version: '2.0.0', code: 'code' }],
        },
      ],
    }

    expect(
      new CarePlanActivityBuilder()
        .use(carePlanActivity)
        .getOutcomeCodeableConcept()
    ).to.eql([
      {
        coding: [
          {
            system: 'system',
            version: '2.0.0',
            code: 'code',
          },
        ],
      },
    ])
  })

  it('should return empty array when outcomeCodeableConcept-array is empty', () => {
    expect(new CarePlanActivityBuilder().getOutcomeCodeableConcept()).to.eql([])
  })

  it('should be able to add CodeableConcept to outcomeCodeableConcept-array', () => {
    expect(
      new CarePlanActivityBuilder()
        .addOutcomeCodeableConcept({
          coding: [
            {
              system: 'system',
              version: '2.0.0',
              code: 'code',
            },
          ],
        })
        .getOutcomeCodeableConcept()
    ).to.eql([
      {
        coding: [
          {
            system: 'system',
            version: '2.0.0',
            code: 'code',
          },
        ],
      },
    ])
  })

  it('should be able to set outcomeReference-array', () => {
    const reference: Reference = new ReferenceBuilder()
      .setReference('#reference')
      .build()

    const carePlanActivity: CarePlanActivity = new CarePlanActivityBuilder()
      .setOutcomeReference([reference])
      .build()

    expect(carePlanActivity).to.eql({
      outcomeReference: [
        {
          reference: '#reference',
        },
      ],
    })
  })

  it('should be able to unset outcomeReference-array', () => {
    const carePlanActivityBuilder: CarePlanActivityBuilder = new CarePlanActivityBuilder().use(
      {
        outcomeReference: [
          {
            reference: '#reference',
          },
        ],
      }
    )

    expect(carePlanActivityBuilder.setOutcomeReference().build()).to.eql({})
  })

  it('should be able to return outcomeReference-array', () => {
    const carePlanActivity: CarePlanActivity = {
      outcomeReference: [
        {
          reference: '#reference',
        },
      ],
    }

    expect(
      new CarePlanActivityBuilder().use(carePlanActivity).getOutcomeReference()
    ).to.eql([
      {
        reference: '#reference',
      },
    ])
  })

  it('should return empty array when outcomeReference-array is empty', () => {
    expect(new CarePlanActivityBuilder().getOutcomeReference()).to.eql([])
  })

  it('should be able to add Reference to outcomeReference-array', () => {
    expect(
      new CarePlanActivityBuilder()
        .addOutcomeReference({
          reference: '#reference',
        })
        .getOutcomeReference()
    ).to.eql([
      {
        reference: '#reference',
      },
    ])
  })

  it('should be able to set reference', () => {
    const reference: Reference = new ReferenceBuilder()
      .setReference('#reference')
      .build()

    const carePlanActivity: CarePlanActivity = new CarePlanActivityBuilder()
      .setReference(reference)
      .build()

    expect(carePlanActivity).to.eql({
      reference: {
        reference: '#reference',
      },
    })
  })

  it('should be able to unset reference', () => {
    const carePlanActivityBuilder: CarePlanActivityBuilder = new CarePlanActivityBuilder().use(
      {
        reference: {
          reference: '#reference',
        },
      }
    )

    expect(carePlanActivityBuilder.setReference().build()).to.eql({})
  })

  it('should be able to get outcomeCodeableConcept-array', () => {
    const carePlanActivity: CarePlanActivity = {
      reference: {
        reference: '#reference',
      },
    }

    expect(
      new CarePlanActivityBuilder().use(carePlanActivity).getReference()
    ).to.eql({
      reference: '#reference',
    })
  })

  it('should be able to tell that two activities are in conflict if they reference the same resource', () => {
    expect(
      new CarePlanActivityBuilder()
        .setReference(new ReferenceBuilder().setReference('#ref').build())
        .conflictsWith(
          new CarePlanActivityBuilder()
            .setReference(new ReferenceBuilder().setReference('#ref').build())
            .build()
        )
    ).to.eql(true)
  })

  it('should be able to tell that two activities are in conflict if they reference the same thresholdSet', () => {
    expect(
      new CarePlanActivityBuilder()
        .addExtension(
          new ExtensionBuilder()
            .setUrl(Constants.carePlan.thresholdSet)
            .setValueReference(
              new ReferenceBuilder().setReference('#ref').build()
            )
            .build()
        )
        .conflictsWith(
          new CarePlanActivityBuilder()
            .addExtension(
              new ExtensionBuilder()
                .setUrl(Constants.carePlan.thresholdSet)
                .setValueReference(
                  new ReferenceBuilder().setReference('#ref').build()
                )
                .build()
            )
            .build()
        )
    ).to.eql(true)
  })
})
