import { expect } from 'chai'
import { Extension, CodeableConcept, Coding } from 'fhir'
import { CodeableConceptBuilder, ExtensionBuilder } from '../src'

describe('CodeableConceptBuilder', () => {
  it('should be able to build minimal CodeableConcept', () => {
    const codeableConcept: CodeableConcept = new CodeableConceptBuilder().build()
    expect(codeableConcept).to.eql({})
  })

  it('should be able to accept existing CodeableConcept', () => {
    const codeableConcept: CodeableConcept = {
      coding: [
        {
          system: 'system',
          code: 'code',
        },
      ],
    }

    expect(new CodeableConceptBuilder().use(codeableConcept).build()).to.eql({
      coding: [
        {
          system: 'system',
          code: 'code',
        },
      ],
    })
  })

  it('should be able to reset', () => {
    const codeableConcept: CodeableConcept = {
      coding: [
        {
          system: 'system',
          code: 'code',
        },
      ],
    }

    const codeableConceptBuilder: CodeableConceptBuilder = new CodeableConceptBuilder().use(
      codeableConcept
    )
    expect(codeableConceptBuilder.use().build()).to.eql({})
  })

  it('should be able to set text-field', () => {
    expect(new CodeableConceptBuilder().setText('text').build()).to.eql({
      text: 'text',
    })
  })

  it('should be able to unset text-field', () => {
    const codeableConceptBuilder: CodeableConceptBuilder = new CodeableConceptBuilder().use(
      {
        text: 'text',
      }
    )

    expect(codeableConceptBuilder.setText().build()).to.eql({})
  })

  it('should be able to return text-field', () => {
    expect(new CodeableConceptBuilder().use({ text: 'text' }).getText()).to.eql(
      'text'
    )
  })

  it('should be able to set coding-array', () => {
    const coding: Coding = { system: 'system', code: 'code' }

    expect(new CodeableConceptBuilder().setCoding([coding]).build()).to.eql({
      coding: [{ system: 'system', code: 'code' }],
    })
  })

  it('should be able to unset coding-array', () => {
    const codeableConceptBuilder: CodeableConceptBuilder = new CodeableConceptBuilder().use(
      {
        coding: [{ system: 'system', code: 'code' }],
      }
    )

    expect(codeableConceptBuilder.setCoding().build()).to.eql({})
  })

  it('should be able to return coding-array', () => {
    const codeableConcept: CodeableConcept = {
      coding: [{ system: 'system', code: 'code' }],
    }

    expect(
      new CodeableConceptBuilder().use(codeableConcept).getCoding()
    ).to.eql([{ system: 'system', code: 'code' }])
  })

  it('should be able to return coding with specific system from coding-array', () => {
    const codeableConcept: CodeableConcept = {
      coding: [
        { system: 'system', code: 'code' },
        { system: 'another-system', code: 'another-code' },
      ],
    }
    expect(
      new CodeableConceptBuilder().use(codeableConcept).getCoding('system')
    ).to.eql({ system: 'system', code: 'code' })
  })

  it('should return empty array when coding-array is empty', () => {
    expect(new CodeableConceptBuilder().getCoding()).to.eql([])
  })

  it('should be able to add coding to coding-array', () => {
    expect(
      new CodeableConceptBuilder()
        .addCoding({ system: 'system', code: 'code' })
        .getCoding()
    ).to.eql([{ system: 'system', code: 'code' }])
  })
})
