import { expect } from 'chai'
import { Extension, DomainResource, Task, Resource } from 'fhir'
import { DomainResourceBuilder, ExtensionBuilder, TaskBuilder } from '../src'
import { AlreadyExistsError } from '../src/exceptions'

describe('DomainResourceBuilder', () => {
  it('should be able to build minimal DomainResource', () => {
    const domainResource: DomainResource = new DomainResourceBuilder().build()
    expect(domainResource).to.eql({
      resourceType: 'DomainResource',
    })
  })

  it('should be able to accept existing DomainResource', () => {
    const domainResource: DomainResource = {
      resourceType: 'DomainResource',
      modifierExtension: [{ url: 'http://example.com/extension' }],
    }

    expect(new DomainResourceBuilder().use(domainResource).build()).to.eql({
      resourceType: 'DomainResource',
      modifierExtension: [{ url: 'http://example.com/extension' }],
    })
  })

  it('should be able to reset', () => {
    const domainResource: DomainResource = {
      resourceType: 'DomainResource',
      modifierExtension: [{ url: 'http://example.com/extension' }],
    }

    const domainResourceBuilder: DomainResourceBuilder = new DomainResourceBuilder().use(
      domainResource
    )
    expect(domainResourceBuilder.use().build()).to.eql({
      resourceType: 'DomainResource',
    })
  })

  it('should be able to set text-field', () => {
    expect(
      new DomainResourceBuilder()
        .setText({
          status: 'additional',
          div: '<b>div</b>',
        })
        .build()
    ).to.eql({
      resourceType: 'DomainResource',
      text: {
        status: 'additional',
        div: '<b>div</b>',
      },
    })
  })

  it('should be able to unset text-field', () => {
    const domainResourceBuilder: DomainResourceBuilder = new DomainResourceBuilder().use(
      {
        resourceType: 'DomainResource',
        text: {
          status: 'additional',
          div: '<b>div</b>',
        },
      }
    )

    expect(domainResourceBuilder.setText().build()).to.eql({
      resourceType: 'DomainResource',
    })
  })

  it('should be able to return text-field', () => {
    expect(
      new DomainResourceBuilder()
        .use({
          resourceType: 'DomainResource',
          text: {
            status: 'additional',
            div: '<b>div</b>',
          },
        })
        .getText()
    ).to.eql({
      status: 'additional',
      div: '<b>div</b>',
    })
  })

  it('should be able to set contained-array', () => {
    const task: Task = new TaskBuilder().setId('7').build()

    const domainResource: DomainResource = new DomainResourceBuilder()
      .setContained([task])
      .build()

    expect(domainResource).to.eql({
      resourceType: 'DomainResource',
      contained: [{ resourceType: 'Task', id: '7' }],
    })
  })

  it('should be able to unset contained-array', () => {
    const domainResourceBuilder: DomainResourceBuilder = new DomainResourceBuilder().use(
      {
        resourceType: 'DomainResource',
        contained: [
          {
            resourceType: 'Task',
            id: '7',
            status: 'requested',
            intent: 'order',
          } as Resource,
        ],
      }
    )

    expect(domainResourceBuilder.setContained().build()).to.eql({
      resourceType: 'DomainResource',
    })
  })

  it('should be able to return contained-array', () => {
    const domainResource: DomainResource = {
      resourceType: 'DomainResource',
      contained: [
        { resourceType: 'Task', id: '7', status: 'requested' } as Task,
      ],
    }

    expect(
      new DomainResourceBuilder().use(domainResource).getContained()
    ).to.eql([{ resourceType: 'Task', id: '7', status: 'requested' }])
  })

  it('should return empty array when contained-array is empty', () => {
    expect(new DomainResourceBuilder().getContained()).to.eql([])
  })

  it('should be able to return resource with specific id from contained-array', () => {
    const domainResource: DomainResource = {
      contained: [
        { resourceType: 'Task', id: '7', status: 'requested' } as Task,
        { resourceType: 'Task', id: '8', status: 'requested' } as Task,
      ],
    }

    expect(
      new DomainResourceBuilder().use(domainResource).getContained('7')
    ).to.eql({ resourceType: 'Task', id: '7', status: 'requested' })
  })

  it('should be able to add resource to contained-array', () => {
    expect(
      new DomainResourceBuilder()
        .addContained({
          resourceType: 'Task',
          id: '7',
          status: 'requested',
        } as Task)
        .getContained()
    ).to.eql([{ resourceType: 'Task', id: '7', status: 'requested' }])
  })

  it('should throw an error when adding an item with an id that already exists in the contained array', () => {
    const domainResource: DomainResource = {
      contained: [
        { resourceType: 'Task', id: '7', status: 'requested' } as Task,
      ],
    }
    expect(() => {
      new DomainResourceBuilder().use(domainResource).addContained({
        resourceType: 'Task',
        id: '7',
        status: 'requested',
      } as Task)
    }).to.throw(AlreadyExistsError)
  })

  it('should be able to set extension-array', () => {
    const extension: Extension = new ExtensionBuilder()
      .setUrl('http://example.com/extension')
      .build()

    const domainResource: DomainResource = new DomainResourceBuilder()
      .setExtension([extension])
      .build()

    expect(domainResource).to.eql({
      resourceType: 'DomainResource',
      extension: [{ url: 'http://example.com/extension' }],
    })
  })

  it('should be able to unset extension-array', () => {
    const domainResourceBuilder: DomainResourceBuilder = new DomainResourceBuilder().use(
      {
        resourceType: 'DomainResource',
        extension: [{ url: 'http://example.com/extension' }],
      }
    )

    expect(domainResourceBuilder.setExtension().build()).to.eql({
      resourceType: 'DomainResource',
    })
  })

  it('should be able to return extension-array', () => {
    const domainResource: DomainResource = {
      resourceType: 'DomainResource',
      extension: [{ url: 'http://example.com/extension' }],
    }

    expect(
      new DomainResourceBuilder().use(domainResource).getExtension()
    ).to.eql([{ url: 'http://example.com/extension' }])
  })

  it('should return empty array when extension-array is empty', () => {
    expect(new DomainResourceBuilder().getExtension()).to.eql([])
  })

  it('should be able to return extensions with specific url from extension-array', () => {
    const domainResource: DomainResource = {
      extension: [
        { url: 'http://example.com/extension' },
        { url: 'http://example.com/another-extension' },
      ],
    }

    expect(
      new DomainResourceBuilder()
        .use(domainResource)
        .getExtension('http://example.com/extension')
    ).to.eql([{ url: 'http://example.com/extension' }])
  })

  it('should be able to add extension to empty extension-array', () => {
    expect(
      new DomainResourceBuilder()
        .addExtension({ url: 'http://example.com/extension' })
        .getExtension()
    ).to.eql([{ url: 'http://example.com/extension' }])
  })

  it('should be able to set modifierExtension-array', () => {
    const extension: Extension = new ExtensionBuilder()
      .setUrl('http://example.com/extension')
      .build()

    const domainResource: DomainResource = new DomainResourceBuilder()
      .setModifierExtension([extension])
      .build()

    expect(domainResource).to.eql({
      resourceType: 'DomainResource',
      modifierExtension: [{ url: 'http://example.com/extension' }],
    })
  })

  it('should be able to unset modifierExtension-array', () => {
    const domainResourceBuilder: DomainResourceBuilder = new DomainResourceBuilder().use(
      {
        resourceType: 'DomainResource',
        modifierExtension: [{ url: 'http://example.com/extension' }],
      }
    )

    expect(domainResourceBuilder.setModifierExtension().build()).to.eql({
      resourceType: 'DomainResource',
    })
  })

  it('should be able to return modifierExtension-array', () => {
    const domainResource: DomainResource = {
      resourceType: 'DomainResource',
      modifierExtension: [{ url: 'http://example.com/extension' }],
    }

    expect(
      new DomainResourceBuilder().use(domainResource).getModifierExtension()
    ).to.eql([{ url: 'http://example.com/extension' }])
  })

  it('should return empty array when modifierExtension-array is empty', () => {
    expect(new DomainResourceBuilder().getModifierExtension()).to.eql([])
  })

  it('should be able to return extensions with specific url from modifierExtension-array', () => {
    const domainResource: DomainResource = {
      modifierExtension: [
        { url: 'http://example.com/extension' },
        { url: 'http://example.com/another-extension' },
      ],
    }

    expect(
      new DomainResourceBuilder()
        .use(domainResource)
        .getModifierExtension('http://example.com/extension')
    ).to.eql([{ url: 'http://example.com/extension' }])
  })

  it('should be able to add extension to empty modifierExtension-array', () => {
    expect(
      new DomainResourceBuilder()
        .addModifierExtension({ url: 'http://example.com/extension' })
        .getModifierExtension()
    ).to.eql([{ url: 'http://example.com/extension' }])
  })
})
