import { expect } from 'chai'
import { ExtensionBuilder } from '../src'
import { Extension } from 'fhir'

describe('ExtensionBuilder', () => {
  it('should be able to build minimal Extension', () => {
    expect(new ExtensionBuilder().build()).to.eql({})
  })

  it('should be able to accept existing Extension', () => {
    expect(
      new ExtensionBuilder()
        .use({
          url: 'http://example.com/extension',
        })
        .build()
    ).to.eql({
      url: 'http://example.com/extension',
    })
  })

  it('should be able to reset', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extension',
    })
    expect(extensionBuilder.use().build()).to.eql({})
  })

  it('should be able to set url-field', () => {
    expect(
      new ExtensionBuilder().setUrl('http://example.com/url').build()
    ).to.eql({
      url: 'http://example.com/url',
    })
  })

  it('should be able to unset url-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/url',
    })

    expect(extensionBuilder.setUrl().build()).to.eql({})
  })

  it('should be able to return url-field', () => {
    expect(
      new ExtensionBuilder().use({ url: 'http://example.com/url' }).getUrl()
    ).to.eql('http://example.com/url')
  })

  it('should be able to set valueBase64Binary-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueBase64Binary(' a G  V   y   b  2 5neWFuZy5jb20=   ')
        .build()
    ).to.eql({
      valueBase64Binary: ' a G  V   y   b  2 5neWFuZy5jb20=   ',
    })
  })

  it('should be able to unset valueBase64Binary-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueBase64Binary: ' a G  V   y   b  2 5neWFuZy5jb20=   ',
    })

    expect(extensionBuilder.setValueBase64Binary().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueBase64Binary-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueBase64Binary: ' a G  V   y   b  2 5neWFuZy5jb20=   ',
        } as Extension)
        .getValueBase64Binary()
    ).to.eql(' a G  V   y   b  2 5neWFuZy5jb20=   ')
  })

  it('should be able to set valueBoolean-field', () => {
    expect(new ExtensionBuilder().setValueBoolean(true).build()).to.eql({
      valueBoolean: true,
    })
  })

  it('should be able to unset valueBoolean-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueBoolean: true,
    })

    expect(extensionBuilder.setValueBoolean().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueBoolean-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueBoolean: true } as Extension)
        .getValueBoolean()
    ).to.eql(true)
  })

  it('should be able to set valueCanonical-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueCanonical(
          'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0'
        )
        .build()
    ).to.eql({
      valueCanonical:
        'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0',
    })
  })

  it('should be able to unset valueCanonical-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueCanonical:
        'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0',
    })

    expect(extensionBuilder.setValueCanonical().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueCanonical-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueCanonical:
            'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0',
        } as Extension)
        .getValueCanonical()
    ).to.eql(
      'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0'
    )
  })

  it('should be able to set valueCode-field', () => {
    expect(new ExtensionBuilder().setValueCode('code').build()).to.eql({
      valueCode: 'code',
    })
  })

  it('should be able to unset valueCode-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueCode: 'code',
    })

    expect(extensionBuilder.setValueCode().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueCode-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueCode: 'code' } as Extension)
        .getValueCode()
    ).to.eql('code')
  })

  it('should be able to set valueDate-field', () => {
    expect(new ExtensionBuilder().setValueDate('1951-06-04').build()).to.eql({
      valueDate: '1951-06-04',
    })
  })

  it('should be able to unset valueDate-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueDate: '1951-06-04',
    })

    expect(extensionBuilder.setValueDate().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueDate-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueDate: '1951-06-04' } as Extension)
        .getValueDate()
    ).to.eql('1951-06-04')
  })

  it('should be able to set valueDateTime-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueDateTime('2013-06-08T10:57:34+01:00')
        .build()
    ).to.eql({
      valueDateTime: '2013-06-08T10:57:34+01:00',
    })
  })

  it('should be able to unset valueDateTime-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueDateTime: '2013-06-08T10:57:34+01:00',
    })

    expect(extensionBuilder.setValueDateTime().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueDateTime-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueDateTime: '2013-06-08T10:57:34+01:00' } as Extension)
        .getValueDateTime()
    ).to.eql('2013-06-08T10:57:34+01:00')
  })

  it('should be able to set valueDecimal-field', () => {
    expect(new ExtensionBuilder().setValueDecimal(22.5).build()).to.eql({
      valueDecimal: 22.5,
    })
  })

  it('should be able to unset valueDecimal-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueDecimal: 22.5,
    })

    expect(extensionBuilder.setValueDecimal().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueDecimal-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueDecimal: 22.5 } as Extension)
        .getValueDecimal()
    ).to.eql(22.5)
  })

  it('should be able to set valueId-field', () => {
    expect(new ExtensionBuilder().setValueId('thresholdSet1').build()).to.eql({
      valueId: 'thresholdSet1',
    })
  })

  it('should be able to unset valueId-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueId: 'thresholdSet1',
    })

    expect(extensionBuilder.setValueId().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueId-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueId: 'thresholdSet1' } as Extension)
        .getValueId()
    ).to.eql('thresholdSet1')
  })

  it('should be able to set valueInstant-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueInstant('2013-06-08T09:57:34.2112Z')
        .build()
    ).to.eql({
      valueInstant: '2013-06-08T09:57:34.2112Z',
    })
  })

  it('should be able to unset valueInstant-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueInstant: '2013-06-08T09:57:34.2112Z',
    })

    expect(extensionBuilder.setValueInstant().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueInstant-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueInstant: '2013-06-08T09:57:34.2112Z' } as Extension)
        .getValueInstant()
    ).to.eql('2013-06-08T09:57:34.2112Z')
  })

  it('should be able to set valueInteger-field', () => {
    expect(new ExtensionBuilder().setValueInteger(3).build()).to.eql({
      valueInteger: 3,
    })
  })

  it('should be able to unset valueInteger-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueInteger: 3,
    })

    expect(extensionBuilder.setValueInteger().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueInteger-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueInteger: 3 } as Extension)
        .getValueInteger()
    ).to.eql(3)
  })

  it('should be able to set valueMarkdown-field', () => {
    expect(
      new ExtensionBuilder().setValueMarkdown('__markdown__').build()
    ).to.eql({
      valueMarkdown: '__markdown__',
    })
  })

  it('should be able to unset valueMarkdown-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueMarkdown: '__markdown__',
    })

    expect(extensionBuilder.setValueMarkdown().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueMarkdown-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueMarkdown: '__markdown__' } as Extension)
        .getValueMarkdown()
    ).to.eql('__markdown__')
  })

  it('should be able to set valueOid-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueOid('urn:oid:1.2.840.10004.1.1.1.0.0.1.0.0.1.2680')
        .build()
    ).to.eql({
      valueOid: 'urn:oid:1.2.840.10004.1.1.1.0.0.1.0.0.1.2680',
    })
  })

  it('should be able to unset valueOid-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueOid: 'urn:oid:1.2.840.10004.1.1.1.0.0.1.0.0.1.2680',
    })

    expect(extensionBuilder.setValueOid().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueOid-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueOid: 'urn:oid:1.2.840.10004.1.1.1.0.0.1.0.0.1.2680',
        } as Extension)
        .getValueOid()
    ).to.eql('urn:oid:1.2.840.10004.1.1.1.0.0.1.0.0.1.2680')
  })

  it('should be able to set valuePositiveInt-field', () => {
    expect(new ExtensionBuilder().setValuePositiveInt(3).build()).to.eql({
      valuePositiveInt: 3,
    })
  })

  it('should be able to unset valuePositiveInt-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valuePositiveInt: 3,
    })

    expect(extensionBuilder.setValuePositiveInt().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valuePositiveInt-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valuePositiveInt: 3 } as Extension)
        .getValuePositiveInt()
    ).to.eql(3)
  })

  it('should be able to set valueString-field', () => {
    expect(new ExtensionBuilder().setValueString('string').build()).to.eql({
      valueString: 'string',
    })
  })

  it('should be able to unset valueString-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueString: 'string',
    })

    expect(extensionBuilder.setValueString().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueString-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueString: 'string' } as Extension)
        .getValueString()
    ).to.eql('string')
  })

  it('should be able to set valueTime-field', () => {
    expect(new ExtensionBuilder().setValueTime('14:35').build()).to.eql({
      valueTime: '14:35',
    })
  })

  it('should be able to unset valueTime-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueTime: '14:35',
    })

    expect(extensionBuilder.setValueTime().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueTime-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueTime: '14:35' } as Extension)
        .getValueTime()
    ).to.eql('14:35')
  })

  it('should be able to set valueUnsignedInt-field', () => {
    expect(new ExtensionBuilder().setValueUnsignedInt(3).build()).to.eql({
      valueUnsignedInt: 3,
    })
  })

  it('should be able to unset valueUnsignedInt-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueUnsignedInt: 3,
    })

    expect(extensionBuilder.setValueUnsignedInt().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueUnsignedInt-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueUnsignedInt: 3 } as Extension)
        .getValueUnsignedInt()
    ).to.eql(3)
  })

  it('should be able to set valueUri-field', () => {
    expect(
      new ExtensionBuilder().setValueUri('urn:iso:std:iso:11073:10101').build()
    ).to.eql({
      valueUri: 'urn:iso:std:iso:11073:10101',
    })
  })

  it('should be able to unset valueUri-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueUri: 'urn:iso:std:iso:11073:10101',
    })

    expect(extensionBuilder.setValueUri().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueUri-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueUri: 'urn:iso:std:iso:11073:10101' } as Extension)
        .getValueUri()
    ).to.eql('urn:iso:std:iso:11073:10101')
  })

  it('should be able to set valueUrl-field', () => {
    expect(
      new ExtensionBuilder().setValueUrl('http://example.com').build()
    ).to.eql({
      valueUrl: 'http://example.com',
    })
  })

  it('should be able to unset valueUrl-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueUrl: 'http://example.com',
    })

    expect(extensionBuilder.setValueUrl().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueUrl-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueUrl: 'http://example.com' } as Extension)
        .getValueUrl()
    ).to.eql('http://example.com')
  })

  it('should be able to set valueUuid-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueUuid('39268cb0-efb8-4e17-9b5b-9dfc7799495c')
        .build()
    ).to.eql({
      valueUuid: '39268cb0-efb8-4e17-9b5b-9dfc7799495c',
    })
  })

  it('should be able to unset valueUuid-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueUuid: '39268cb0-efb8-4e17-9b5b-9dfc7799495c',
    })

    expect(extensionBuilder.setValueUuid().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueUuid-field', () => {
    expect(
      new ExtensionBuilder()
        .use({ valueUuid: '39268cb0-efb8-4e17-9b5b-9dfc7799495c' } as Extension)
        .getValueUuid()
    ).to.eql('39268cb0-efb8-4e17-9b5b-9dfc7799495c')
  })

  it('should be able to set valueAddress-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueAddress({
          use: 'home',
          type: 'postal',
          text: 'Mainstreet 12',
          line: ['Defaultcity'],
          district: 'Defaultcity',
          country: 'STD',
        })
        .build()
    ).to.eql({
      valueAddress: {
        use: 'home',
        type: 'postal',
        text: 'Mainstreet 12',
        line: ['Defaultcity'],
        district: 'Defaultcity',
        country: 'STD',
      },
    })
  })

  it('should be able to unset valueAddress-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueAddress: {
        use: 'home',
        type: 'postal',
        text: 'Mainstreet 12',
        line: ['Defaultcity'],
        district: 'Defaultcity',
        country: 'STD',
      },
    })

    expect(extensionBuilder.setValueAddress().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueAddress-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueAddress: {
            use: 'home',
            type: 'postal',
            text: 'Mainstreet 12',
            line: ['Defaultcity'],
            district: 'Defaultcity',
            country: 'STD',
          },
        } as Extension)
        .getValueAddress()
    ).to.eql({
      use: 'home',
      type: 'postal',
      text: 'Mainstreet 12',
      line: ['Defaultcity'],
      district: 'Defaultcity',
      country: 'STD',
    })
  })

  it('should be able to set valueAge-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueAge({
          value: 12,
          unit: 'years',
        })
        .build()
    ).to.eql({
      valueAge: {
        value: 12,
        unit: 'years',
      },
    })
  })

  it('should be able to unset valueAge-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueAge: {
        value: 12,
        unit: 'years',
      },
    })

    expect(extensionBuilder.setValueAge().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueAge-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueAge: {
            value: 12,
            unit: 'years',
          },
        } as Extension)
        .getValueAge()
    ).to.eql({
      value: 12,
      unit: 'years',
    })
  })

  it('should be able to set valueAnnotation-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueAnnotation({
          text: 'An annotation',
        })
        .build()
    ).to.eql({
      valueAnnotation: {
        text: 'An annotation',
      },
    })
  })

  it('should be able to unset valueAnnotation-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueAnnotation: {
        text: 'An annotation',
      },
    })

    expect(extensionBuilder.setValueAnnotation().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueAnnotation-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueAnnotation: {
            text: 'An annotation',
          },
        } as Extension)
        .getValueAnnotation()
    ).to.eql({
      text: 'An annotation',
    })
  })

  it('should be able to set valueAttachment-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueAttachment({
          title: 'Attachment',
        })
        .build()
    ).to.eql({
      valueAttachment: {
        title: 'Attachment',
      },
    })
  })

  it('should be able to unset valueAttachment-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueAttachment: {
        title: 'Attachment',
      },
    })

    expect(extensionBuilder.setValueAttachment().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueAttachment-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueAttachment: {
            title: 'Attachment',
          },
        } as Extension)
        .getValueAttachment()
    ).to.eql({
      title: 'Attachment',
    })
  })

  it('should be able to set valueCodeableConcept-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueCodeableConcept({
          coding: [
            {
              system: 'system',
              code: 'code',
            },
          ],
          text: 'CodeableConcept',
        })
        .build()
    ).to.eql({
      valueCodeableConcept: {
        coding: [
          {
            system: 'system',
            code: 'code',
          },
        ],
        text: 'CodeableConcept',
      },
    })
  })

  it('should be able to unset valueCodeableConcept-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueCodeableConcept: {
        coding: [
          {
            system: 'system',
            code: 'code',
          },
        ],
        text: 'CodeableConcept',
      },
    })

    expect(extensionBuilder.setValueCodeableConcept().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueCodeableConcept-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueCodeableConcept: {
            coding: [
              {
                system: 'system',
                code: 'code',
              },
            ],
            text: 'CodeableConcept',
          },
        } as Extension)
        .getValueCodeableConcept()
    ).to.eql({
      coding: [
        {
          system: 'system',
          code: 'code',
        },
      ],
      text: 'CodeableConcept',
    })
  })

  it('should be able to set valueCoding-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueCoding({
          system: 'system',
          code: 'code',
        })
        .build()
    ).to.eql({
      valueCoding: {
        system: 'system',
        code: 'code',
      },
    })
  })

  it('should be able to unset valueCoding-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueCoding: {
        system: 'system',
        code: 'code',
      },
    })

    expect(extensionBuilder.setValueCoding().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueCoding-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueCoding: {
            system: 'system',
            code: 'code',
          },
        } as Extension)
        .getValueCoding()
    ).to.eql({
      system: 'system',
      code: 'code',
    })
  })

  it('should be able to set valueContactPoint-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueContactPoint({
          system: 'phone',
          value: '555-0123',
          use: 'home',
        })
        .build()
    ).to.eql({
      valueContactPoint: {
        system: 'phone',
        value: '555-0123',
        use: 'home',
      },
    })
  })

  it('should be able to unset valueContactPoint-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueContactPoint: {
        system: 'phone',
        value: '555-0123',
        use: 'home',
      },
    })

    expect(extensionBuilder.setValueContactPoint().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueContactPoint-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueContactPoint: {
            system: 'phone',
            value: '555-0123',
            use: 'home',
          },
        } as Extension)
        .getValueContactPoint()
    ).to.eql({
      system: 'phone',
      value: '555-0123',
      use: 'home',
    })
  })

  it('should be able to set valueCount-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueCount({
          value: 15,
          unit: 'times',
        })
        .build()
    ).to.eql({
      valueCount: {
        value: 15,
        unit: 'times',
      },
    })
  })

  it('should be able to unset valueCount-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueCount: {
        value: 15,
        unit: 'times',
      },
    })

    expect(extensionBuilder.setValueCount().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueCount-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueCount: {
            value: 15,
            unit: 'times',
          },
        } as Extension)
        .getValueCount()
    ).to.eql({
      value: 15,
      unit: 'times',
    })
  })

  it('should be able to set valueDistance-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueDistance({
          value: 15,
          unit: 'km',
        })
        .build()
    ).to.eql({
      valueDistance: {
        value: 15,
        unit: 'km',
      },
    })
  })

  it('should be able to unset valueDistance-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueDistance: {
        value: 15,
        unit: 'km',
      },
    })

    expect(extensionBuilder.setValueDistance().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueDistance-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueDistance: {
            value: 15,
            unit: 'km',
          },
        } as Extension)
        .getValueDistance()
    ).to.eql({
      value: 15,
      unit: 'km',
    })
  })

  it('should be able to set valueDuration-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueDuration({
          value: 15,
          unit: 'seconds',
        })
        .build()
    ).to.eql({
      valueDuration: {
        value: 15,
        unit: 'seconds',
      },
    })
  })

  it('should be able to unset valueDuration-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueDuration: {
        value: 15,
        unit: 'seconds',
      },
    })

    expect(extensionBuilder.setValueDuration().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueDuration-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueDuration: {
            value: 15,
            unit: 'seconds',
          },
        } as Extension)
        .getValueDuration()
    ).to.eql({
      value: 15,
      unit: 'seconds',
    })
  })

  it('should be able to set valueHumanName-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueHumanName({
          use: 'official',
          family: 'Berggren',
          given: ['Nancy', 'Ann', 'Test'],
        })
        .build()
    ).to.eql({
      valueHumanName: {
        use: 'official',
        family: 'Berggren',
        given: ['Nancy', 'Ann', 'Test'],
      },
    })
  })

  it('should be able to unset valueHumanName-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueHumanName: {
        use: 'official',
        family: 'Berggren',
        given: ['Nancy', 'Ann', 'Test'],
      },
    })

    expect(extensionBuilder.setValueHumanName().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueHumanName-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueHumanName: {
            use: 'official',
            family: 'Berggren',
            given: ['Nancy', 'Ann', 'Test'],
          },
        } as Extension)
        .getValueHumanName()
    ).to.eql({
      use: 'official',
      family: 'Berggren',
      given: ['Nancy', 'Ann', 'Test'],
    })
  })

  it('should be able to set valueIdentifier-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueIdentifier({
          system: 'system',
          value: 'value',
        })
        .build()
    ).to.eql({
      valueIdentifier: {
        system: 'system',
        value: 'value',
      },
    })
  })

  it('should be able to unset valueIdentifier-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueIdentifier: {
        system: 'system',
        value: 'value',
      },
    })

    expect(extensionBuilder.setValueIdentifier().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueIdentifier-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueIdentifier: {
            system: 'system',
            value: 'value',
          },
        } as Extension)
        .getValueIdentifier()
    ).to.eql({
      system: 'system',
      value: 'value',
    })
  })

  it('should be able to set valueMoney-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueMoney({
          value: 150000,
          currency: 'CHF',
        })
        .build()
    ).to.eql({
      valueMoney: {
        value: 150000,
        currency: 'CHF',
      },
    })
  })

  it('should be able to unset valueMoney-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueMoney: {
        value: 150000,
        currency: 'CHF',
      },
    })

    expect(extensionBuilder.setValueMoney().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueMoney-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueMoney: {
            value: 150000,
            currency: 'CHF',
          },
        } as Extension)
        .getValueMoney()
    ).to.eql({
      value: 150000,
      currency: 'CHF',
    })
  })

  it('should be able to set valuePeriod-field', () => {
    expect(
      new ExtensionBuilder()
        .setValuePeriod({
          start: '1951-06-04',
          end: '1953-08-02',
        })
        .build()
    ).to.eql({
      valuePeriod: {
        start: '1951-06-04',
        end: '1953-08-02',
      },
    })
  })

  it('should be able to unset valuePeriod-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valuePeriod: {
        start: '1951-06-04',
        end: '1953-08-02',
      },
    })

    expect(extensionBuilder.setValuePeriod().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valuePeriod-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valuePeriod: {
            start: '1951-06-04',
            end: '1953-08-02',
          },
        } as Extension)
        .getValuePeriod()
    ).to.eql({
      start: '1951-06-04',
      end: '1953-08-02',
    })
  })

  it('should be able to set valueQuantity-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueQuantity({
          value: 250.3,
          system: 'http://unitsofmeasure.org ',
          code: 'kg',
        })
        .build()
    ).to.eql({
      valueQuantity: {
        value: 250.3,
        system: 'http://unitsofmeasure.org ',
        code: 'kg',
      },
    })
  })

  it('should be able to unset valueQuantity-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueQuantity: {
        value: 250.3,
        system: 'http://unitsofmeasure.org ',
        code: 'kg',
      },
    })

    expect(extensionBuilder.setValueQuantity().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueQuantity-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueQuantity: {
            value: 250.3,
            system: 'http://unitsofmeasure.org ',
            code: 'kg',
          },
        } as Extension)
        .getValueQuantity()
    ).to.eql({
      value: 250.3,
      system: 'http://unitsofmeasure.org ',
      code: 'kg',
    })
  })

  it('should be able to set valueRange-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueRange({
          low: {
            value: 17,
            unit: 'kg',
          },
          high: {
            value: 33,
            unit: 'kg',
          },
        })
        .build()
    ).to.eql({
      valueRange: {
        low: {
          value: 17,
          unit: 'kg',
        },
        high: {
          value: 33,
          unit: 'kg',
        },
      },
    })
  })

  it('should be able to unset valueRange-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueRange: {
        low: {
          value: 17,
          unit: 'kg',
        },
        high: {
          value: 33,
          unit: 'kg',
        },
      },
    })

    expect(extensionBuilder.setValueRange().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueRange-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueRange: {
            low: {
              value: 17,
              unit: 'kg',
            },
            high: {
              value: 33,
              unit: 'kg',
            },
          },
        } as Extension)
        .getValueRange()
    ).to.eql({
      low: {
        value: 17,
        unit: 'kg',
      },
      high: {
        value: 33,
        unit: 'kg',
      },
    })
  })

  it('should be able to set valueRatio-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueRatio({
          numerator: {
            value: 2,
          },
          denominator: {
            value: 7,
          },
        })
        .build()
    ).to.eql({
      valueRatio: {
        numerator: {
          value: 2,
        },
        denominator: {
          value: 7,
        },
      },
    })
  })

  it('should be able to unset valueRatio-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueRatio: {
        numerator: {
          value: 2,
        },
        denominator: {
          value: 7,
        },
      },
    })

    expect(extensionBuilder.setValueRatio().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueRatio-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueRatio: {
            numerator: {
              value: 2,
            },
            denominator: {
              value: 7,
            },
          },
        } as Extension)
        .getValueRatio()
    ).to.eql({
      numerator: {
        value: 2,
      },
      denominator: {
        value: 7,
      },
    })
  })

  it('should be able to set valueReference-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueReference({
          reference: '#task1',
          type: 'Task',
        })
        .build()
    ).to.eql({
      valueReference: {
        reference: '#task1',
        type: 'Task',
      },
    })
  })

  it('should be able to unset valueReference-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueReference: {
        reference: '#task1',
        type: 'Task',
      },
    })

    expect(extensionBuilder.setValueReference().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueReference-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueReference: {
            reference: '#task1',
            type: 'Task',
          },
        } as Extension)
        .getValueReference()
    ).to.eql({
      reference: '#task1',
      type: 'Task',
    })
  })

  it('should be able to set valueSampledData-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueSampledData({
          origin: {
            value: 0.0,
            unit: 'bpm',
            system: 'http://unitsofmeasure.org',
            code: '/min',
          },
          period: 250.0,
          factor: 0.25,
          dimensions: 1,
          data:
            '323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324',
        })
        .build()
    ).to.eql({
      valueSampledData: {
        origin: {
          value: 0.0,
          unit: 'bpm',
          system: 'http://unitsofmeasure.org',
          code: '/min',
        },
        period: 250.0,
        factor: 0.25,
        dimensions: 1,
        data:
          '323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324',
      },
    })
  })

  it('should be able to unset valueSampledData-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueSampledData: {
        origin: {
          value: 0.0,
          unit: 'bpm',
          system: 'http://unitsofmeasure.org',
          code: '/min',
        },
        period: 250.0,
        factor: 0.25,
        dimensions: 1,
        data:
          '323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324',
      },
    })

    expect(extensionBuilder.setValueSampledData().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueSampledData-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueSampledData: {
            origin: {
              value: 0.0,
              unit: 'bpm',
              system: 'http://unitsofmeasure.org',
              code: '/min',
            },
            period: 250.0,
            factor: 0.25,
            dimensions: 1,
            data:
              '323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324',
          },
        } as Extension)
        .getValueSampledData()
    ).to.eql({
      origin: {
        value: 0.0,
        unit: 'bpm',
        system: 'http://unitsofmeasure.org',
        code: '/min',
      },
      period: 250.0,
      factor: 0.25,
      dimensions: 1,
      data:
        '323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324 323 322 322 322 323 323 322 322 323 324',
    })
  })

  it('should be able to set valueSignature-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueSignature({
          type: [
            {
              system: 'urn:iso-astm:E1762-95:2013',
              code: '1.2.840.10065.1.12.1.1',
            },
          ],
          when: '2013-06-08T09:57:34.2112Z',
          targetFormat: 'image/bmp',
          sigFormat: 'image/bmp',
        })
        .build()
    ).to.eql({
      valueSignature: {
        type: [
          {
            system: 'urn:iso-astm:E1762-95:2013',
            code: '1.2.840.10065.1.12.1.1',
          },
        ],
        when: '2013-06-08T09:57:34.2112Z',
        targetFormat: 'image/bmp',
        sigFormat: 'image/bmp',
      },
    })
  })

  it('should be able to unset valueSignature-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueSignature: {
        type: [
          {
            system: 'urn:iso-astm:E1762-95:2013',
            code: '1.2.840.10065.1.12.1.1',
          },
        ],
        when: '2013-06-08T09:57:34.2112Z',
        targetFormat: 'image/bmp',
        sigFormat: 'image/bmp',
      },
    })

    expect(extensionBuilder.setValueSignature().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueSignature-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueSignature: {
            type: [
              {
                system: 'urn:iso-astm:E1762-95:2013',
                code: '1.2.840.10065.1.12.1.1',
              },
            ],
            when: '2013-06-08T09:57:34.2112Z',
            targetFormat: 'image/bmp',
            sigFormat: 'image/bmp',
          },
        } as Extension)
        .getValueSignature()
    ).to.eql({
      type: [
        {
          system: 'urn:iso-astm:E1762-95:2013',
          code: '1.2.840.10065.1.12.1.1',
        },
      ],
      when: '2013-06-08T09:57:34.2112Z',
      targetFormat: 'image/bmp',
      sigFormat: 'image/bmp',
    })
  })

  it('should be able to set valueTiming-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueTiming({
          event: [
            '2012-01-07T09:00:00+10:00',
            '2012-01-14T09:00:00+10:00',
            '2012-01-22T11:00:00+10:00',
          ],
        })
        .build()
    ).to.eql({
      valueTiming: {
        event: [
          '2012-01-07T09:00:00+10:00',
          '2012-01-14T09:00:00+10:00',
          '2012-01-22T11:00:00+10:00',
        ],
      },
    })
  })

  it('should be able to unset valueSignature-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueTiming: {
        event: [
          '2012-01-07T09:00:00+10:00',
          '2012-01-14T09:00:00+10:00',
          '2012-01-22T11:00:00+10:00',
        ],
      },
    })

    expect(extensionBuilder.setValueTiming().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueTiming-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueTiming: {
            event: [
              '2012-01-07T09:00:00+10:00',
              '2012-01-14T09:00:00+10:00',
              '2012-01-22T11:00:00+10:00',
            ],
          },
        } as Extension)
        .getValueTiming()
    ).to.eql({
      event: [
        '2012-01-07T09:00:00+10:00',
        '2012-01-14T09:00:00+10:00',
        '2012-01-22T11:00:00+10:00',
      ],
    })
  })

  it('should be able to set valueMeta-field', () => {
    expect(
      new ExtensionBuilder()
        .setValueMeta({
          lastUpdated: '2019-06-24T14:00:00.000Z',
          profile: ['http://hl7.org/fhir/uv/phd/StructureDefinition/PhgDevice'],
        })
        .build()
    ).to.eql({
      valueMeta: {
        lastUpdated: '2019-06-24T14:00:00.000Z',
        profile: ['http://hl7.org/fhir/uv/phd/StructureDefinition/PhgDevice'],
      },
    })
  })

  it('should be able to unset valueMeta-field', () => {
    const extensionBuilder: ExtensionBuilder = new ExtensionBuilder().use({
      url: 'http://example.com/extensions/ext',
      valueMeta: {
        lastUpdated: '2019-06-24T14:00:00.000Z',
        profile: ['http://hl7.org/fhir/uv/phd/StructureDefinition/PhgDevice'],
      },
    })

    expect(extensionBuilder.setValueMeta().build()).to.eql({
      url: 'http://example.com/extensions/ext',
    })
  })

  it('should be able to return valueMeta-field', () => {
    expect(
      new ExtensionBuilder()
        .use({
          valueMeta: {
            lastUpdated: '2019-06-24T14:00:00.000Z',
            profile: [
              'http://hl7.org/fhir/uv/phd/StructureDefinition/PhgDevice',
            ],
          },
        } as Extension)
        .getValueMeta()
    ).to.eql({
      lastUpdated: '2019-06-24T14:00:00.000Z',
      profile: ['http://hl7.org/fhir/uv/phd/StructureDefinition/PhgDevice'],
    })
  })
})
