import { expect } from 'chai'
import { Reference } from 'fhir'
import { ReferenceBuilder, IdentifierBuilder } from '../src'

describe('ReferenceBuilder', () => {
  it('should be able to build minimal Reference', () => {
    expect(new ReferenceBuilder().build()).to.eql({})
  })

  it('should be able to accept existing Reference', () => {
    expect(
      new ReferenceBuilder()
        .use({
          reference: 'http://example.com/Questionnaire/7',
        })
        .build()
    ).to.eql({
      reference: 'http://example.com/Questionnaire/7',
    })
  })

  it('should be able to reset', () => {
    const referenceBuilder: ReferenceBuilder = new ReferenceBuilder().use({
      reference: 'http://example.com/Questionnaire/7',
    })
    expect(referenceBuilder.use().build()).to.eql({})
  })

  it('should be able to set reference-field', () => {
    expect(
      new ReferenceBuilder()
        .setReference('http://example.com/Questionnaire/7')
        .build()
    ).to.eql({
      reference: 'http://example.com/Questionnaire/7',
    })
  })

  it('should be able to unset reference-field', () => {
    const referenceBuilder: ReferenceBuilder = new ReferenceBuilder().use({
      reference: 'http://example.com/Questionnaire/7',
    })
    expect(referenceBuilder.setReference().build()).to.eql({})
  })

  it('should be able to return reference-field', () => {
    expect(
      new ReferenceBuilder()
        .use({ reference: 'http://example.com/Questionnaire/7' })
        .getReference()
    ).to.eql('http://example.com/Questionnaire/7')
  })

  it('should be able to set type-field', () => {
    expect(new ReferenceBuilder().setType('Questionnaire').build()).to.eql({
      type: 'Questionnaire',
    })
  })

  it('should be able to unset type-field', () => {
    const referenceBuilder: ReferenceBuilder = new ReferenceBuilder().use({
      type: 'Questionnaire',
    })
    expect(referenceBuilder.setType().build()).to.eql({})
  })

  it('should be able to return type-field', () => {
    expect(
      new ReferenceBuilder().use({ type: 'Questionnaire' }).getType()
    ).to.eql('Questionnaire')
  })

  it('should be able to set identifier-field', () => {
    expect(
      new ReferenceBuilder()
        .setIdentifier(
          new IdentifierBuilder()
            .setSystem('system')
            .setValue('value')
            .build()
        )
        .build()
    ).to.eql({
      identifier: {
        system: 'system',
        value: 'value',
      },
    })
  })

  it('should be able to unset identifier-field', () => {
    const referenceBuilder: ReferenceBuilder = new ReferenceBuilder().use({
      identifier: {
        system: 'system',
        value: 'value',
      },
    })
    expect(referenceBuilder.setIdentifier().build()).to.eql({})
  })

  it('should be able to return identifier-field', () => {
    expect(
      new ReferenceBuilder()
        .use({
          identifier: {
            system: 'system',
            value: 'value',
          },
        })
        .getIdentifier()
    ).to.eql({
      system: 'system',
      value: 'value',
    })
  })

  it('should be able to set display-field', () => {
    expect(new ReferenceBuilder().setDisplay('Questionnaire').build()).to.eql({
      display: 'Questionnaire',
    })
  })

  it('should be able to unset display-field', () => {
    const referenceBuilder: ReferenceBuilder = new ReferenceBuilder().use({
      display: 'Questionnaire',
    })
    expect(referenceBuilder.setDisplay().build()).to.eql({})
  })

  it('should be able to return display-field', () => {
    expect(
      new ReferenceBuilder().use({ display: 'Questionnaire' }).getDisplay()
    ).to.eql('Questionnaire')
  })

  it('should be able to tell if reference is in conflict with other reference based on reference-field', () => {
    expect(
      new ReferenceBuilder()
        .use({ reference: 'http://example.com/Questionnaire/8' })
        .conflictsWith({ reference: 'http://example.com/Questionnaire/8' })
    ).to.eql(true)
  })

  it('should be able to tell if reference is in conflict with other reference based on identifier-field', () => {
    expect(
      new ReferenceBuilder()
        .use({
          identifier: {
            system: 'system',
            value: 'value',
          },
        })
        .conflictsWith({
          identifier: {
            system: 'system',
            value: 'value',
          },
        })
    ).to.eql(true)
  })

  it('should be able to tell if reference is in not conflict with other reference', () => {
    expect(
      new ReferenceBuilder()
        .use({
          identifier: {
            system: 'system',
            value: 'value',
          },
        })
        .conflictsWith({
          identifier: {
            system: 'system',
            value: 'other-value',
          },
        })
    ).to.eql(false)
  })

  it('should say two references are not in conflict of one of them is incomplete', () => {
    expect(
      new ReferenceBuilder()
        .use({
          identifier: {
            system: 'system',
            value: 'value',
          },
        })
        .conflictsWith({
          identifier: {
            system: 'system',
          },
        })
    ).to.eql(false)
  })
})
