import { expect } from 'chai'
import { TaskInputBuilder, CodeableConceptBuilder, CodingBuilder } from '../src'
import { AlreadyExistsError } from '../src/exceptions'

describe('TaskInputBuilder', () => {
  it('should be able to build minimal TaskInput', () => {
    expect(new TaskInputBuilder().build()).to.eql({})
  })

  it('should be able to accept existing TaskInput', () => {
    expect(
      new TaskInputBuilder()
        .use({
          id: 'id',
          type: {
            coding: [
              {
                system: 'system',
                code: 'code',
              },
            ],
          },
          valueCanonical:
            'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0',
        })
        .build()
    ).to.eql({
      id: 'id',
      type: {
        coding: [
          {
            system: 'system',
            code: 'code',
          },
        ],
      },
      valueCanonical:
        'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0',
    })
  })

  it('should be able to reset', () => {
    const taskBuilder: TaskInputBuilder = new TaskInputBuilder().use({
      id: 'id',
      type: {
        coding: [
          {
            system: 'system',
            code: 'code',
          },
        ],
      },
      valueCanonical:
        'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0',
    })
    expect(taskBuilder.use().build()).to.eql({})
  })

  it('should be able to set type-field', () => {
    expect(
      new TaskInputBuilder()
        .setType({
          coding: [
            {
              system: 'system',
              code: 'code',
            },
          ],
        })
        .build()
    ).to.eql({
      type: {
        coding: [
          {
            system: 'system',
            code: 'code',
          },
        ],
      },
    })
  })

  it('should be able to unset type-field', () => {
    const inputBuilder: TaskInputBuilder = new TaskInputBuilder().use({
      type: {
        coding: [
          {
            system: 'system',
            code: 'code',
          },
        ],
      },
    })

    expect(inputBuilder.setType().build()).to.eql({})
  })

  it('should be able to return type-field', () => {
    expect(
      new TaskInputBuilder()
        .use({
          type: {
            coding: [
              {
                system: 'system',
                code: 'code',
              },
            ],
          },
        })
        .getType()
    ).to.eql({
      coding: [
        {
          system: 'system',
          code: 'code',
        },
      ],
    })
  })

  it('should be able to set valueCanonical-field', () => {
    expect(
      new TaskInputBuilder()
        .setValueCanonical(
          'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0'
        )
        .build()
    ).to.eql({
      valueCanonical:
        'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0',
    })
  })

  it('should be able to unset valueCanonical-field', () => {
    const inputBuilder: TaskInputBuilder = new TaskInputBuilder().use({
      type: {
        coding: [
          {
            system: 'system',
            code: 'code',
          },
        ],
      },
      valueCanonical:
        'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0',
    })

    expect(inputBuilder.setValueCanonical().build()).to.eql({
      type: {
        coding: [
          {
            system: 'system',
            code: 'code',
          },
        ],
      },
    })
  })

  it('should be able to return valueCanonical-field', () => {
    expect(
      new TaskInputBuilder()
        .use({
          type: {
            coding: [
              {
                system: 'system',
                code: 'code',
              },
            ],
          },
          valueCanonical:
            'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0',
        })
        .getValueCanonical()
    ).to.eql(
      'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0'
    )
  })

  it('should be able to set valueReference-field', () => {
    expect(
      new TaskInputBuilder()
        .setValueReference({
          reference: '#ref',
        })
        .build()
    ).to.eql({
      valueReference: {
        reference: '#ref',
      },
    })
  })

  it('should be able to unset valueReference-field', () => {
    const inputBuilder: TaskInputBuilder = new TaskInputBuilder().use({
      type: {
        coding: [
          {
            system: 'system',
            code: 'code',
          },
        ],
      },
      valueReference: {
        reference: '#ref',
      },
    })

    expect(inputBuilder.setValueReference().build()).to.eql({
      type: {
        coding: [
          {
            system: 'system',
            code: 'code',
          },
        ],
      },
    })
  })

  it('should be able to return valueReference-field', () => {
    expect(
      new TaskInputBuilder()
        .use({
          type: {
            coding: [
              {
                system: 'system',
                code: 'code',
              },
            ],
          },
          valueReference: {
            reference: '#ref',
          },
        })
        .getValueReference()
    ).to.eql({
      reference: '#ref',
    })
  })

  it('should be able to tell that two inputs with same valueCanonical are in conflict', () => {
    expect(
      new TaskInputBuilder()
        .setId('id')
        .setType(
          new CodeableConceptBuilder()
            .setCoding([
              new CodingBuilder()
                .setSystem('system')
                .setCode('code')
                .build(),
            ])
            .build()
        )
        .setValueCanonical(
          'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0'
        )
        .conflictsWith(
          new TaskInputBuilder()
            .setId('id')
            .setType(
              new CodeableConceptBuilder()
                .setCoding([
                  new CodingBuilder()
                    .setSystem('system')
                    .setCode('code')
                    .build(),
                ])
                .build()
            )
            .setValueCanonical(
              'canonical://uuid/Questionnaire/39268cb0-efb8-4e17-9b5b-9dfc7799495c|1.0.0'
            )
            .build()
        )
    ).to.eql(true)
  })

  it('should be able to tell that two inputs with same valueReference are in conflict', () => {
    expect(
      new TaskInputBuilder()
        .setId('id')
        .setType(
          new CodeableConceptBuilder()
            .setCoding([
              new CodingBuilder()
                .setSystem('system')
                .setCode('code')
                .build(),
            ])
            .build()
        )
        .setValueReference({
          reference: '#ref',
        })
        .conflictsWith(
          new TaskInputBuilder()
            .setId('id')
            .setType(
              new CodeableConceptBuilder()
                .setCoding([
                  new CodingBuilder()
                    .setSystem('system')
                    .setCode('code')
                    .build(),
                ])
                .build()
            )
            .setValueReference({
              reference: '#ref',
            })
            .build()
        )
    ).to.eql(true)
  })
})
