import { expect } from 'chai'
import carePlan from './resources/CarePlan.json'
import { CarePlan } from 'fhir'
import { deepcopy } from './utils'
import { extractThresholdSetsFromCarePlan } from '../src/utils/threshold-set-extractor'

describe('extractThresholdSetsFromCarePlan', () => {
  let testCarePlan: CarePlan | undefined

  beforeEach(() => {
    testCarePlan = deepcopy(carePlan) as CarePlan
  })

  it('should be able to retrieve all ThresholdSets from a CarePlan', () => {
    expect(extractThresholdSetsFromCarePlan(testCarePlan as CarePlan)).to.eql([
      {
        resourceType: 'Basic',
        id: 'thresholdSetb',
        identifier: [
          {
            system: 'urn:ietf:rfc:3986',
            value: '2b3dfcb1-0312-4248-a437-49c63ccb1c7e',
          },
        ],
        meta: {
          profile: ['http://example.com/StructureDefinition/thresholdset'],
        },
        extension: [
          {
            url:
              'https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100499511/ThresholdSet+version+extension',
            valueString: '1.0.0',
          },
          {
            url:
              'https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100433991/ThresholdSet+status+extension',
            valueCode: 'active',
          },
          {
            url:
              'https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100433995/ThresholdSet+approvalDate+extension',
            valueDateTime: '2019-03-28T13:05:03+01:00',
          },
          {
            url:
              'https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100565062/ThresholdSet+approvingPractitioner+extension',
            valueReference: {
              type: 'Practitioner',
              identifier: {
                system:
                  'https://stps.dk/da/sundhedsprofessionelle-og-myndigheder/autorisationsregister/autorisationsid/',
                value: '55GT7',
              },
            },
          },
          {
            url:
              'https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100728906/ThresholdSet+appliesTo+extension',
            valueCanonical:
              'canonical://uuid/Questionnaire/f14997cf-e16e-4e4b-8b3a-1f419386f597|1.0.0',
          },
          {
            url:
              'https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100499515/ThresholdSet+threshold+extension',
            extension: [
              {
                url: 'http://example.com/thresholdset_threshold_type_extension',
                valueCodeableConcept: {
                  coding: [
                    {
                      system:
                        'https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100728891/ThresholdSet+type+coding+system',
                      code: 'pathological',
                      display: 'Pathological threshold',
                    },
                  ],
                  text: 'Pathological threshold',
                },
              },
              {
                url:
                  'https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100728921/ThresholdSet+effectiveWhen+extension',
                extension: [
                  {
                    url:
                      'https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100728925/ThresholdSet+component+extension',
                    valueCoding: {
                      system:
                        'https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100433987/ThresholdSet+component+coding+system',
                      code: 'systolic',
                    },
                  },
                  {
                    url:
                      'https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100565090/ThresholdSet+value+extension',
                    valueInteger: 140,
                  },
                  {
                    url:
                      'https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100499530/ThresholdSet+operator+extension',
                    valueCode: '>',
                  },
                ],
              },
              {
                url:
                  'https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100728931/ThresholdSet+effectiveBehavior+extension',
                valueCode: 'any',
              },
            ],
          },
        ],
        code: {
          coding: [
            {
              system:
                'https://issuetracker4s.atlassian.net/wiki/spaces/FHIRReg/pages/100728935/Custom+FHIR+Resources+coding+system',
              code: 'thresholdset',
              display: 'ThresholdSet',
            },
          ],
          text: 'ThresholdSet',
        },
      },
    ])
  })

  afterEach(() => {
    testCarePlan = undefined
  })
})
