import { expect } from 'chai'
import { ThresholdSetThresholdBuilder } from 'threshold-set-threshold-builder'
import { Constants } from 'global/constants'
import { MalformedError } from 'exceptions/malformed-error'
import { Extension } from 'fhir'

describe('ThresholdSetThresholdBuilder', () => {
  it('should be able to build minimal threshold', () => {
    expect(new ThresholdSetThresholdBuilder().build()).to.eql({
      url: Constants.thresholdSet.threshold,
      extension: [],
    })
  })

  it('should be able to accept existing threshold', () => {
    expect(
      new ThresholdSetThresholdBuilder()
        .use({
          url: Constants.thresholdSet.threshold,
          extension: [
            {
              url: Constants.thresholdSet.type,
              valueCodeableConcept: {
                text: 'text',
              },
            },
          ],
        })
        .build()
    ).to.eql({
      url: Constants.thresholdSet.threshold,
      extension: [
        {
          url: Constants.thresholdSet.type,
          valueCodeableConcept: {
            text: 'text',
          },
        },
      ],
    })
  })

  it('should be able to reset', () => {
    const thresholdBuilder: ThresholdSetThresholdBuilder = new ThresholdSetThresholdBuilder().use(
      {
        url: Constants.thresholdSet.threshold,
        extension: [
          {
            url: Constants.thresholdSet.type,
            valueCodeableConcept: {
              text: 'text',
            },
          },
        ],
      }
    )
    expect(thresholdBuilder.use().build()).to.eql({
      url: Constants.thresholdSet.threshold,
      extension: [],
    })
  })

  it('should be able to set type-field', () => {
    expect(
      new ThresholdSetThresholdBuilder()
        .setType({
          text: 'text',
        })
        .build()
    ).to.eql({
      url: Constants.thresholdSet.threshold,
      extension: [
        {
          url: Constants.thresholdSet.type,
          valueCodeableConcept: {
            text: 'text',
          },
        },
      ],
    })
  })

  it('should be able to unset type-field', () => {
    const thresholdBuilder: ThresholdSetThresholdBuilder = new ThresholdSetThresholdBuilder().use(
      {
        url: Constants.thresholdSet.threshold,
        extension: [
          {
            url: Constants.thresholdSet.type,
            valueCodeableConcept: {
              text: 'text',
            },
          },
        ],
      }
    )

    expect(thresholdBuilder.setType().build()).to.eql({
      url: Constants.thresholdSet.threshold,
      extension: [],
    })
  })

  it('should be able to return type-field', () => {
    expect(
      new ThresholdSetThresholdBuilder()
        .use({
          url: Constants.thresholdSet.threshold,
          extension: [
            {
              url: Constants.thresholdSet.type,
              valueCodeableConcept: {
                text: 'text',
              },
            },
          ],
        })
        .getType()
    ).to.eql({
      text: 'text',
    })
  })

  it('should throw error if type-extension is malformed or there are more than one', () => {
    expect(() => {
      new ThresholdSetThresholdBuilder()
        .use({
          url: Constants.thresholdSet.threshold,
          extension: [
            {
              url: Constants.thresholdSet.type,
              valueCodeableConcept: {
                text: 'text',
              },
            },
          ],
        })
        .addExtension({
          url: Constants.thresholdSet.type,
          valueCodeableConcept: {
            text: 'text',
          },
        })
        .getType()
    }).to.throw(MalformedError)
  })

  it('should return undefined if type-extension is not set', () => {
    expect(
      new ThresholdSetThresholdBuilder()
        .use({
          url: Constants.thresholdSet.threshold,
          extension: [],
        })
        .getType()
    ).to.be.an('undefined')
  })

  it('should be able to set effectiveWhen-array', () => {
    const effectiveWhen: Extension = {
      url: Constants.thresholdSet.effectiveWhen,
      extension: [],
    }

    expect(
      new ThresholdSetThresholdBuilder()
        .setEffectiveWhen([effectiveWhen])
        .build()
    ).to.eql({
      url: Constants.thresholdSet.threshold,
      extension: [
        {
          url: Constants.thresholdSet.effectiveWhen,
          extension: [],
        },
      ],
    })
  })

  it('should be able to unset threshold-array', () => {
    const thresholdBuilder: ThresholdSetThresholdBuilder = new ThresholdSetThresholdBuilder().use(
      {
        url: Constants.thresholdSet.threshold,
        extension: [
          {
            url: Constants.thresholdSet.effectiveWhen,
            extension: [],
          },
        ],
      }
    )

    expect(thresholdBuilder.setEffectiveWhen().build()).to.eql({
      url: Constants.thresholdSet.threshold,
      extension: [],
    })
  })

  it('should be able to return effectiveWhen-array', () => {
    const threshold: Extension = {
      url: Constants.thresholdSet.threshold,
      extension: [
        {
          url: Constants.thresholdSet.effectiveWhen,
          extension: [],
        },
      ],
    }

    expect(
      new ThresholdSetThresholdBuilder().use(threshold).getEffectiveWhen()
    ).to.eql([
      {
        url: Constants.thresholdSet.effectiveWhen,
        extension: [],
      },
    ])
  })

  it('should return empty array when effectiveWhen-array is empty', () => {
    expect(new ThresholdSetThresholdBuilder().getEffectiveWhen()).to.eql([])
  })

  it('should be able to add resource to effectiveWhen-array', () => {
    expect(
      new ThresholdSetThresholdBuilder()
        .addEffectiveWhen({
          url: Constants.thresholdSet.effectiveWhen,
          extension: [],
        })
        .getEffectiveWhen()
    ).to.eql([
      {
        url: Constants.thresholdSet.effectiveWhen,
        extension: [],
      },
    ])
  })

  it('should be able to set effectiveBehavior-field', () => {
    expect(
      new ThresholdSetThresholdBuilder().setEffectiveBehavior('all').build()
    ).to.eql({
      url: Constants.thresholdSet.threshold,
      extension: [
        {
          url: Constants.thresholdSet.effectiveBehavior,
          valueCode: 'all',
        },
      ],
    })
  })

  it('should be able to unset effectiveBehavior-field', () => {
    const thresholdBuilder: ThresholdSetThresholdBuilder = new ThresholdSetThresholdBuilder().use(
      {
        url: Constants.thresholdSet.threshold,
        extension: [
          {
            url: Constants.thresholdSet.effectiveBehavior,
            valueCode: 'all',
          },
        ],
      }
    )

    expect(thresholdBuilder.setEffectiveBehavior().build()).to.eql({
      url: Constants.thresholdSet.threshold,
      extension: [],
    })
  })

  it('should be able to return effectiveBehavior-field', () => {
    expect(
      new ThresholdSetThresholdBuilder()
        .use({
          url: Constants.thresholdSet.threshold,
          extension: [
            {
              url: Constants.thresholdSet.effectiveBehavior,
              valueCode: 'all',
            },
          ],
        })
        .getEffectiveBehavior()
    ).to.eql('all')
  })

  it('should throw error if effectiveBehavior-extension is malformed or there are more than one', () => {
    expect(() => {
      new ThresholdSetThresholdBuilder()
        .use({
          url: Constants.thresholdSet.threshold,
          extension: [
            {
              url: Constants.thresholdSet.effectiveBehavior,
              valueCode: 'all',
            },
          ],
        })
        .addExtension({
          url: Constants.thresholdSet.effectiveBehavior,
          valueCode: 'all',
        })
        .getEffectiveBehavior()
    }).to.throw(MalformedError)
  })

  it('should throw error if effectiveBehavior-extension is not one of the allowed values', () => {
    expect(() => {
      new ThresholdSetThresholdBuilder()
        .use({
          url: Constants.thresholdSet.threshold,
          extension: [
            {
              url: Constants.thresholdSet.effectiveBehavior,
              valueCode: 'something-else',
            },
          ],
        })
        .getEffectiveBehavior()
    }).to.throw(MalformedError)
  })

  it('should return undefined if effectiveBehavior-extension is not set', () => {
    expect(new ThresholdSetThresholdBuilder().getEffectiveBehavior()).to.be.an(
      'undefined'
    )
  })
})
